/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-27 14:52:17.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamStudentQuestionScoreInfo;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.base.utils.JsonMapper;
import cn.com.qmth.examcloud.core.oe.common.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordForMarkingEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordForMarkingRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordRepo;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 考试记录相关接口
 *
 * @author: fengdesheng
 * @since: 2018/8/27
 */
@RestController
@Api(tags = "考试记录相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/record")
public class ExamRecordController extends ControllerSupport {
    @Autowired
    private ExamRecordService examRecordService;
    
    @Autowired
    private ExamRecordRepo examRecodRepo;
    
    @Autowired
    private ExamRecordForMarkingRepo examRecordForMarkingRepo;
    

    @PostMapping("/waiting/audit/list")
    @ApiOperation(value = "查询“监考待审”列表(分页)")
    public Page<ExamRecordInfo> getExamRecordWaitingAuditList(@RequestBody ExamRecordQuery query) {
        return examRecordService.getExamRecordWaitingAuditList(query);
    }
    
    @PostMapping("/detail/check")
    @ApiOperation(value = "查询“考试明细” 是否存在待审数据")
    public Long existsWarnExamRecordDetail(@RequestBody ExamRecordQuery query) {
        return examRecordService.existsWarnExamRecordDetail(query);
    }
    
    @PostMapping("/detail/list")
    @ApiOperation(value = "查询“考试明细”列表(分页)")
    public Page<ExamRecordInfo> getExamRecordDetailList(@RequestBody ExamRecordQuery query) {
    	Page<ExamRecordInfo> examRecordInfoPage = examRecordService.getExamRecordDetailListForPage(query);
    	List<ExamRecordInfo> examRecordInfoList = examRecordInfoPage.getContent();
    	if(examRecordInfoList!=null && examRecordInfoList.size() > 0){
    		String examType = examRecordInfoList.get(0).getExamType();
    		if(ExamType.OFFLINE.name() == examType){
    			for(ExamRecordInfo examRecordInfo:examRecordInfoList){
    				long examRecordDataId = examRecordInfo.getDataId();
        			ExamRecordForMarkingEntity examRecordForMarkingEntity = examRecordForMarkingRepo.findByExamRecordDataId(examRecordDataId);
        			if(examRecordForMarkingEntity!=null){
        				examRecordInfo.setOfflineFileUrl(examRecordForMarkingEntity.getOfflineFileUrl());
        			}
    			}
    		}
    	}
        return examRecordInfoPage;
    }

    @Naked
    @GetMapping("/detail/list/export")
    @ApiOperation(value = "导出“考试明细”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamRecordDetailList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamRecordQuery newQuery = new JsonMapper().fromJson(query, ExamRecordQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");
        List<ExamRecordInfo> examRecordInfoList = examRecordService.getExamRecordDetailList(newQuery);
        ExportUtils.exportEXCEL("考试明细列表", ExamRecordInfo.class,examRecordInfoList,response);
    }

    @PostMapping(value = "/refresh/capture/statistic")
    @ApiOperation(value = "监考待审-重新统计", notes = "根据人脸识别阀值重新计算数据")
    public ResponseEntity<String> refreshCaptureStatistic(@RequestParam Long examId) {
        //todo
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PostMapping(value = "/clean/capture/statistic")
    @ApiOperation(value = "监考待审-清理数据")
    public ResponseEntity<String> cleanCaptureStatistic() {
        //todo
        //examRecordService.cleanCaptureStatistic();
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping("/select/byExamStudentId")
    @ApiOperation(value = "根据考生ID查询考试记录")
    public List<ExamRecordEntity> getExamRecordByExamStudentId(@RequestParam Long examStudentId){
    	List<ExamRecordEntity> examRecordEntityList = examRecodRepo.findByExamStudentId(examStudentId);
    	return 	examRecordEntityList;	
    }
    
    @GetMapping("/detail/examStudentQuestionScoreList")
    @ApiOperation(value = "导出考生作答分数明细列表(Excel)", notes = "参数examId和examId必传")
    public void getExamStudentQuestionScoreList(@RequestParam Long examId,@RequestParam String courseCode, HttpServletResponse response) throws Exception {
        Check.isNull(examId, "考试id不能为空！");
        Check.isBlank(courseCode, "课程代码不能空！");
        List<ExamStudentQuestionScoreInfo> examRecordInfoList = examRecordService.getExamStudentQuestionScoreList(examId,courseCode);
        ExportUtils.exportEXCEL("考试明细列表", ExamStudentQuestionScoreInfo.class,examRecordInfoList,response);
        
//        return examRecordInfoList;List<ExamStudentQuestionScoreInfo>
    }
}