/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-16 17:40:53.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentHashMap;

import javax.servlet.http.HttpServletResponse;

import cn.com.qmth.examcloud.core.oe.admin.service.ExamCacheTransferHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.CourseProgressInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentEntityConvert;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentExcel;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentFinishedStatistic;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentOrgStatistic;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentUnFinishedExcel;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.base.utils.JsonMapper;
import cn.com.qmth.examcloud.core.oe.common.base.utils.excel.ExportUtils;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import cn.com.qmth.examcloud.web.support.Naked;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 考生信息接口
 *
 * @author: fengdesheng
 * @since: 2018/8/16
 */
@RestController
@Api(tags = "考生信息相关接口")
@RequestMapping("${$rmp.ctr.oe}/exam/student")
public class ExamStudentController extends ControllerSupport {
    @Autowired
    private ExamStudentService examStudentService;
    @Autowired
    private GainBaseDataService gainBaseDataService;
    
    /**
     * 查询课程本地缓存
     */
    private static Map<Long,CourseBean> courseBeanLocalCache;
    
    private static Timer timer;
    
    static{
    	courseBeanLocalCache = new ConcurrentHashMap<Long, CourseBean>();
    	timer = new Timer();
    }
    
    class CleanCourseBeanCacheTask extends TimerTask{
		@Override
		public void run() {
			courseBeanLocalCache.clear();
		}
    }
    
    public ExamStudentController(){
    	long delay = 10*60*1000;//10分钟后开始
    	long period = 30*60*1000;//30分钟清理一次
    	timer.schedule(new CleanCourseBeanCacheTask(),delay,period);
    }

    @PostMapping("/examScheduling/list")
    @ApiOperation(value = "查询“考试进度详情”列表(分页)")
    public Page<ExamStudentInfo> getExamScheduling(@RequestBody ExamStudentQuery query) {
    	Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "考试批次不能为空！");
        return examStudentService.getExamStudentListPage(query);
    }

    @GetMapping("/examScheduling/list/export")
    @ApiOperation(value = "导出“考试进度详情”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamStudentList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamStudentQuery newQuery = new JsonMapper().fromJson(query, ExamStudentQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");
        List<ExamStudentInfo> examStudentInfoList = examStudentService.getExamStudentInfoList(newQuery);
        List<ExamStudentExcel> list = ExamStudentEntityConvert.ofExcel(examStudentInfoList);
        ExportUtils.exportEXCEL("考试详情列表", ExamStudentExcel.class, list, response);
    }

    @PostMapping("/unfinished/list")
    @ApiOperation(value = "查询“缺考登记”列表(分页)")
    public Page<ExamStudentInfo> getExamStudentUnFinishedList(@RequestBody ExamStudentQuery query) {
    	Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "考试批次不能为空！");
        query.setFinished(0);
        return examStudentService.getExamStudentListPage(query);
    }

    @Naked
    @GetMapping("/unfinished/list/export")
    @ApiOperation(value = "导出“缺考登记”列表(Excel)", notes = "参数示例：query={\"pageNo\":1,\"pageSize\":10,\"examId\":123, ...}")
    public void exportExamStudentUnFinishedList(@RequestParam String query, HttpServletResponse response) throws Exception {
        ExamStudentQuery newQuery = new JsonMapper().fromJson(query, ExamStudentQuery.class);
        Check.isNull(newQuery, "请求参数不能为空！");
        Check.isNull(newQuery.getExamId(), "请先选择考试批次！");
        newQuery.setFinished(0);//未完成
        List<ExamStudentInfo> examStudentInfos = examStudentService.getExamStudentInfoList(newQuery);
        List<ExamStudentUnFinishedExcel> list = ExamStudentEntityConvert.ofUnFinishedExcel(examStudentInfos);
        ExportUtils.exportEXCEL("缺考登记列表", ExamStudentUnFinishedExcel.class, list, response);
    }

    @PostMapping("/reexamine/list")
    @ApiOperation(value = "查询重考考生列表(分页)")
    public Page<ExamStudentInfo> getReExamineStudentList(@RequestBody ExamStudentQuery query) {
    	Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "考试批次不能为空！");
        return examStudentService.getReExamineStudentList(query);
    }
    
    @PostMapping("/reexamine")
    @ApiOperation(value = "设置重考")
    public void setReexamine(@RequestParam Long examStudentId){
    	if(examStudentId == null){
    		return;
    	}
    	examStudentService.setReexamine(examStudentId);
    }

    @PostMapping("/statistic/by/finished")
    @ApiOperation(value = "考试概况-按考试完成进度统计")
    public ExamStudentFinishedStatistic getExamStudentStatisticByFinished(@RequestParam Long examId) {
    	if(examId == null){
    		return null;
    	}
        return examStudentService.getExamStudentStatisticByFinished(examId);
    }

    @PostMapping("/statistic/by/org")
    @ApiOperation(value = "考试概况-按学习中心统计")
    public List<ExamStudentOrgStatistic> getExamStudentStatisticByOrg(@RequestParam Long examId, @RequestParam(required = false) Long orgId) {
    	if(examId == null){
    		return null;
    	}
        return examStudentService.getExamStudentStatisticByOrg(examId, orgId);
    }
    
    /**
     * 查询课程
     * @param examId
     * @param orgId
     * @return
     */
    @GetMapping("/findCoursesByExamIdAndOrgId")
    @ApiOperation(value = "查询课程")
    public List<CourseBean> findCoursesByExamIdAndOrgId(@RequestParam Long examId, @RequestParam(required=false) Long orgId){
    	if(examId == null){
    		return null;
    	}
    	User user = getAccessUser();
    	List<Long> courseIdList = examStudentService.findCoursesFromExamStudent(examId, orgId);
    	List<CourseBean> courseBeanList = new ArrayList<CourseBean>();
    	for(Long courseId:courseIdList){
    		CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);
    		courseBeanList.add(courseBean);
    	}
    	return courseBeanList;
    }
    
    /**
     * 课程完成进度
     * @param examId
     * @param courseId
     * @return
     */
    @GetMapping("/courseProgress/list")
    @ApiOperation(value = "课程完成进度")
    public List<CourseProgressInfo> queryCourseProgressInfos(@RequestParam Long examId,
    														  @RequestParam(required=false) Long courseId,
    														  @RequestParam(required=false) String orderColumn){
    	if(examId == null){
    		return null;
    	}
    	List<CourseProgressInfo> courseProgressInfoList = examStudentService.queryCourseProgressInfos(examId,courseId,orderColumn);
    	if(courseProgressInfoList!=null && courseProgressInfoList.size()>0){
    		for(CourseProgressInfo courseProgressInfo:courseProgressInfoList){
    			long key = courseProgressInfo.getCourseId();
        		CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(key);
        		courseProgressInfo.setCourseCode(courseBean.getCode());
        		courseProgressInfo.setCourseName(courseBean.getName());
        	}
    	}
    	return courseProgressInfoList;
    }
    
    
}