package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordForSelectScore;
import cn.com.qmth.examcloud.core.oe.admin.api.request.CheckExamIsStartedReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetExamPhotoVerifyDataReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.QueryExamRecordForSelectScoreReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.CheckExamIsStartedResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetExamPhotoVerifyDataResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.QueryExamRecordForSelectScoreResp;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamCaptureEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamCaptureRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamScoreRepo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * @author  	chenken
 * @date    	2018年9月19日 下午2:44:12
 * @company 	QMTH
 * @description ExamRecordCloudServiceProvider.java
 */
@Api(tags = "考试记录相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/examRecord")
public class ExamRecordCloudServiceProvider extends ControllerSupport implements ExamRecordCloudService{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 7678785677176906341L;
	
	@Autowired
	private ExamRecordRepo examRecordRepo;
	
	@Autowired
	private ExamScoreRepo examScoreRepo;
	
	@Autowired
	private ExamRecordDataRepo examRecordDataRepo;
	
	@Autowired
	private ExamCaptureRepo examCaptureRepo;

	@Override
	@ApiOperation(value = "查询是否已经开考")
	@PostMapping("/checkExamIsStarted")
	public CheckExamIsStartedResp checkExamIsStarted(@RequestBody CheckExamIsStartedReq examRecordReq) {
		Check.isNull(examRecordReq.getExamId(), "examId不能为空");
		ExamRecordEntity examRecordEntity = new ExamRecordEntity();
		examRecordEntity.setExamId(examRecordReq.getExamId());
		examRecordEntity.setCourseId(examRecordReq.getCourseId());
		examRecordEntity.setStudentId(examRecordReq.getStudentId());
		long examRecordNum = examRecordRepo.count(Example.of(examRecordEntity));
		CheckExamIsStartedResp checkExamIsStartedResp = new CheckExamIsStartedResp();
		checkExamIsStartedResp.setIsStarted(examRecordNum>0);
		return checkExamIsStartedResp;
	}

	@Override
	@ApiOperation(value = "查询考试记录-供exchange调用")
	@PostMapping("/queryExamRecordForSelectScoreByScoreId")
	public QueryExamRecordForSelectScoreResp queryExamRecordForSelectScoreByScoreId(@RequestBody QueryExamRecordForSelectScoreReq req) {
		QueryExamRecordForSelectScoreResp resp = new QueryExamRecordForSelectScoreResp();
		if(req.getExamScoreId() != null){
			ExamScoreEntity examScore =GlobalHelper.getEntity(examScoreRepo,req.getExamScoreId(),ExamScoreEntity.class);
			ExamRecordDataEntity examRecordData =GlobalHelper.getEntity(examRecordDataRepo,examScore.getExamRecordDataId(),ExamRecordDataEntity.class);
			List<ExamRecordForSelectScore> examRecordForSelectScoreList = new ArrayList<ExamRecordForSelectScore>();
			examRecordForSelectScoreList.add(buildExamRecordForSelectScore(examRecordData));
			resp.setExamRecordForSelectScoreList(examRecordForSelectScoreList);
		}
		return resp;
	}
	
	@Override
	@ApiOperation(value = "查询考试记录-供exchange调用")
	@PostMapping("/queryExamRecordForSelectScoreByExamStudentId")
	public QueryExamRecordForSelectScoreResp queryExamRecordForSelectScoreByExamStudentId(@RequestBody QueryExamRecordForSelectScoreReq req) {
		QueryExamRecordForSelectScoreResp resp = new QueryExamRecordForSelectScoreResp();
		if(req.getExamStudentId() != null){
			List<ExamRecordEntity> examRecordList = examRecordRepo.findByExamStudentId(req.getExamStudentId());
			List<ExamRecordForSelectScore> examRecordForSelectScoreList = new ArrayList<ExamRecordForSelectScore>();
			for(ExamRecordEntity examRecord:examRecordList){
				ExamRecordDataEntity examRecordData = examRecordDataRepo.findByExamRecordId(examRecord.getId());
				examRecordForSelectScoreList.add(buildExamRecordForSelectScore(examRecordData));
			}
			resp.setExamRecordForSelectScoreList(examRecordForSelectScoreList);
		}
		return resp;
	}

	public ExamRecordForSelectScore buildExamRecordForSelectScore(ExamRecordDataEntity examRecordData){
		ExamRecordForSelectScore examRecordForSelectScore = new ExamRecordForSelectScore();
		ExamRecordEntity examRecord =GlobalHelper.getEntity(examRecordRepo,examRecordData.getExamRecordId(),ExamRecordEntity.class);
		examRecordForSelectScore.setExamStudentId(examRecord.getExamStudentId());
		examRecordForSelectScore.setRootOrgId(examRecord.getRootOrgId());
		examRecordForSelectScore.setExamRecordDataId(examRecordData.getId());
		examRecordForSelectScore.setStartTime(examRecordData.getStartTime());
		examRecordForSelectScore.setEndTime(examRecordData.getEndTime());
		examRecordForSelectScore.setCleanTime(examRecordData.getCleanTime());
		examRecordForSelectScore.setIsWarn(examRecordData.getIsWarn());
		examRecordForSelectScore.setIsAudit(examRecordData.getIsAudit());
		examRecordForSelectScore.setIsIllegality(examRecordData.getIsIllegality());
		examRecordForSelectScore.setStatus(examRecordData.getExamRecordStatus().name());
		
		return examRecordForSelectScore;
	}

	@Override
	@ApiOperation(value = "查询照片数据-供exchange调用")
	@PostMapping("/getExamPhotoVerifyData")
	public GetExamPhotoVerifyDataResp getExamPhotoVerifyData(@RequestBody GetExamPhotoVerifyDataReq req) {
		Long scoreId = req.getScoreId();
		ExamScoreEntity examScoreEntity =GlobalHelper.getEntity(examScoreRepo,scoreId,ExamScoreEntity.class);
		if(examScoreEntity == null){
			return null;
		}
		Long examRecordDataId = examScoreEntity.getExamRecordDataId();
		ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo,examRecordDataId,ExamRecordDataEntity.class);
		if(examRecordData == null){
			return null;
		}
		List<ExamCaptureEntity> examCaptureList = examCaptureRepo.findByExamRecordDataId(examRecordDataId);
		
		GetExamPhotoVerifyDataResp resp = new GetExamPhotoVerifyDataResp();
		resp.setScoreId(scoreId);
		resp.setSuccessRate(examRecordData.getFaceSuccessPercent());
		int strangerCount = 0;
		List<String> photoUrls = new ArrayList<String>();
		for(ExamCaptureEntity examCaptureEntity:examCaptureList){
			if(examCaptureEntity.getIsStranger()){
				strangerCount++;
			}
			photoUrls.add(examCaptureEntity.getFileUrl());
		}
		resp.setStrangerCount(strangerCount);
		resp.setPhotoUrls(photoUrls);
		return resp;
	}
}
