package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import static cn.com.qmth.examcloud.core.oe.common.base.Constants.OE_CODE_400;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordForMarkingCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordForMarkingBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.FindExamRecordForMarkingInfoReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.QueryValidExamRecordInfoPageReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.QueryValidExamRecordInfoReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.FindExamRecordForMarkingInfoResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.QueryValidExamRecordInfoPageResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.QueryValidExamRecordInfoResp;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordForMarkingService;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordForMarkingEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamStudentRepo;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * @author  	chenken
 * @date    	2018年9月27日 上午11:01:23
 * @company 	QMTH
 * @description 阅卷相关接口
 */
@Api(tags = "阅卷获取信息相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/examRecordForMarking")
public class ExamRecordForMarkingCloudServiceProvider extends ControllerSupport implements ExamRecordForMarkingCloudService{
	
	private static final long serialVersionUID = -8343697288418890873L;
	
	@Autowired
	private ExamRecordForMarkingService examRecordForMarkingService;
	@Autowired
	private ExamStudentRepo examStudentRepo;

	@Override
	@ApiOperation(value = "根据条件查询阅卷需要的信息")
	@PostMapping("/findExamRecordForMarkingInfo")
	public FindExamRecordForMarkingInfoResp findExamRecordForMarkingInfo(@RequestBody FindExamRecordForMarkingInfoReq req) {
		Long id =  req.getId();
		Long examId = req.getExamId();
		FindExamRecordForMarkingInfoResp resp = new FindExamRecordForMarkingInfoResp();
		if(id == null && examId == null){
			return resp;
		}
		List<ExamRecordForMarkingEntity> examRecordForMarkingList = examRecordForMarkingService.findExamRecordForMarkingInfo(id,examId,req.getCourseId());
		List<ExamRecordForMarkingBean> examRecordForMarkingBeanList = new ArrayList<ExamRecordForMarkingBean>();
		for(ExamRecordForMarkingEntity entity:examRecordForMarkingList){
			ExamRecordForMarkingBean examRecordForMarkingBean = new ExamRecordForMarkingBean();
			examRecordForMarkingBean.setId(entity.getId());
			examRecordForMarkingBean.setExamId(entity.getExamId());
			examRecordForMarkingBean.setBasePaperId(entity.getBasePaperId());
			examRecordForMarkingBean.setPaperType(entity.getPaperType());
			examRecordForMarkingBean.setCourseId(entity.getCourseId());
			examRecordForMarkingBean.setOfflineFileName(entity.getOfflineFileName());
			examRecordForMarkingBean.setOfflineFileUrl(entity.getOfflineFileUrl());
			examRecordForMarkingBeanList.add(examRecordForMarkingBean);
		}
		resp.setExamRecordForMarkingBeanList(examRecordForMarkingBeanList);
		return resp;
	}

	@Override
	@ApiOperation(value = "查询有效成绩")
	@PostMapping("/queryValidExamRecordInfo")
	public QueryValidExamRecordInfoResp queryValidExamRecordInfo(@RequestBody QueryValidExamRecordInfoReq req) {
		Check.isNull(req.getExamId(), "examId不能为空");
		Check.isNull(req.getCourseId(), "courseId不能为空");
		List<ExamRecordForMarkingEntity> examRecordForMarkingList = examRecordForMarkingService.queryValidExamRecordInfo(req.getExamId(), req.getCourseId());
		
		QueryValidExamRecordInfoResp resp = new QueryValidExamRecordInfoResp();
		List<ExamRecordForMarkingBean> examRecordForMarkingBeanList = new ArrayList<ExamRecordForMarkingBean>();
		for(ExamRecordForMarkingEntity entity:examRecordForMarkingList){
			ExamRecordForMarkingBean examRecordForMarkingBean = new ExamRecordForMarkingBean();
			examRecordForMarkingBean.setId(entity.getId());
			examRecordForMarkingBean.setExamId(entity.getExamId());
			examRecordForMarkingBean.setExamRecordDataId(entity.getExamRecordDataId());
			examRecordForMarkingBean.setExamStudentId(entity.getExamStudentId());
			examRecordForMarkingBean.setBasePaperId(entity.getBasePaperId());
			examRecordForMarkingBean.setPaperType(entity.getPaperType());
			examRecordForMarkingBean.setCourseId(entity.getCourseId());
			examRecordForMarkingBean.setObjectiveScore(entity.getObjectiveScore());
			examRecordForMarkingBeanList.add(examRecordForMarkingBean);
		}
		resp.setExamRecordForMarkingBeanList(examRecordForMarkingBeanList);
		return resp;
	}

	@Override
	@ApiOperation(value = "分页查询有效成绩")
	@PostMapping("/queryValidExamRecordInfoPage")
	public QueryValidExamRecordInfoPageResp queryValidExamRecordInfoPage(@RequestBody QueryValidExamRecordInfoPageReq req) {
		Check.isNull(req.getExamId(), "examId不能为空");
		Check.isNull(req.getCourseId(), "courseId不能为空");
		Check.isNull(req.getStart(), "start不能为空");
		Check.isNull(req.getSize(), "size不能为空");
		if (req.getStart().longValue()<=0) {
            throw new StatusException(OE_CODE_400, "start必须大于0");
        }
		if (req.getSize().longValue()<=0) {
			throw new StatusException(OE_CODE_400, "size必须大于0");
        }
		Long courseId=req.getCourseId();
		Long examId = req.getExamId();
		Long size=req.getSize();
		Long start =req.getStart();
		//分页获取考生id
		Pageable pageable = PageRequest.of(0, size.intValue(), Sort.Direction.ASC, "id");

		Specification<ExamStudentEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("examId"), examId));
			predicates.add(cb.equal(root.get("courseId"), courseId));
			predicates.add(cb.greaterThanOrEqualTo(root.get("id"), start));
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Page<ExamStudentEntity> page = examStudentRepo.findAll(specification,
				pageable);

		Iterator<ExamStudentEntity> iterator = page.iterator();

		List<Long> stuIdList = Lists.newArrayList();
		Long next = start;
		while (iterator.hasNext()) {
			ExamStudentEntity e = iterator.next();
			next = e.getId();
			stuIdList.add(e.getExamStudentId());
		}

		QueryValidExamRecordInfoPageResp resp = new QueryValidExamRecordInfoPageResp();
		if (!next .equals(start)) {
			next++;
		}
		resp.setNext(next);
		//根据分页获取的考生id获取需要阅卷的试卷
		List<ExamRecordForMarkingBean> examRecordForMarkingBeanList = new ArrayList<ExamRecordForMarkingBean>();
		if(stuIdList.size()>0) {
			List<ExamRecordForMarkingEntity> examRecordForMarkingList = examRecordForMarkingService.queryValidExamRecordInfoByStuIds(examId, courseId,stuIdList);
			for(ExamRecordForMarkingEntity entity:examRecordForMarkingList){
				ExamRecordForMarkingBean examRecordForMarkingBean = new ExamRecordForMarkingBean();
				examRecordForMarkingBean.setId(entity.getId());
				examRecordForMarkingBean.setExamId(entity.getExamId());
				examRecordForMarkingBean.setExamRecordDataId(entity.getExamRecordDataId());
				examRecordForMarkingBean.setExamStudentId(entity.getExamStudentId());
				examRecordForMarkingBean.setBasePaperId(entity.getBasePaperId());
				examRecordForMarkingBean.setPaperType(entity.getPaperType());
				examRecordForMarkingBean.setCourseId(entity.getCourseId());
				examRecordForMarkingBean.setObjectiveScore(entity.getObjectiveScore());
				examRecordForMarkingBeanList.add(examRecordForMarkingBean);
			}
		}
		resp.setExamRecordForMarkingBeanList(examRecordForMarkingBeanList);
		return resp;
	}

}
