package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.core.oe.admin.api.ExamScoreObtainQueueCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.request.DeleteExamScoreQueueReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.GetTopExamScoreQueueReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.GetTopExamScoreQueueResp;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamScoreObtainQueueEntity;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamScoreObtainQueueRepo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(tags = "考试分数队列接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/examScoreObtainQueue")
public class ExamScoreObtainQueueCloudServiceProvider extends ControllerSupport implements ExamScoreObtainQueueCloudService{

	/**
	 * 
	 */
	private static final long serialVersionUID = 9204539038969246524L;
	
	@Autowired
	private ExamScoreObtainQueueRepo examScoreObtainQueueRepo;
	
	@Override
	@ApiOperation(value = "获取当前机构分数队列顶层记录")
	@PostMapping("/getTopExamScoreQueue")
	public GetTopExamScoreQueueResp getTopExamScoreQueue(@RequestBody GetTopExamScoreQueueReq req) {
		Check.isNull(req.getRootOrgId(), "顶级机构ID不能为空");
		ExamScoreObtainQueueEntity examScoreObtainQueue = examScoreObtainQueueRepo.findTopByRootOrgIdAndIsValidOrderByIdAsc(req.getRootOrgId(), true);
		GetTopExamScoreQueueResp getTopExamScoreQueueResp = new GetTopExamScoreQueueResp();
		getTopExamScoreQueueResp.setIsEmpty(true);
		if(examScoreObtainQueue!=null){
			getTopExamScoreQueueResp.setExamId(examScoreObtainQueue.getExamId());
			getTopExamScoreQueueResp.setQueueId(examScoreObtainQueue.getId());
			getTopExamScoreQueueResp.setExamRecordDataId(examScoreObtainQueue.getExamRecordDataId());
			getTopExamScoreQueueResp.setIsEmpty(false);
		}
		return getTopExamScoreQueueResp;
	}

	@Override
	@ApiOperation(value = "删除分数队列记录")
	@PostMapping("/deleteExamScoreQueue")
	public void deleteExamScoreQueue(@RequestBody DeleteExamScoreQueueReq req) {
		Check.isNull(req.getQueueId(), "队列ID不能为空");
		Check.isNull(req.getRootOrgId(), "顶级机构ID不能为空");
		ExamScoreObtainQueueEntity examScoreObtainQueue =GlobalHelper.getEntity(examScoreObtainQueueRepo,req.getQueueId(),ExamScoreObtainQueueEntity.class);
		if(examScoreObtainQueue!=null && examScoreObtainQueue.getRootOrgId().equals(req.getRootOrgId())){
			examScoreObtainQueue.setIsValid(false);
			examScoreObtainQueueRepo.save(examScoreObtainQueue);
		}
	}

}
