/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-15 14:15:05.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import java.util.ArrayList;
import java.util.List;

import cn.com.qmth.examcloud.core.oe.admin.api.request.UnbindExamStudentReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.UnbindExamStudentResp;
import com.mysql.cj.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.OeExamStudentCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.FindStudentType;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.OeExamStudentBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.ExamStudentSyncAllDataReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.ExamStudentSyncPartDataReq;
import cn.com.qmth.examcloud.core.oe.admin.api.request.FindExamStudentInfoReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.ExamStudentSyncAllDataResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.ExamStudentSyncPartDataResp;
import cn.com.qmth.examcloud.core.oe.admin.api.response.FindExamStudentInfoResp;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamStudentRepo;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 考生信息接口
 *
 * @author: fengdesheng
 * @since: 2018/8/15
 */
@Api(tags = "考生信息相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/exam/student")
public class ExamStudentCloudServiceProvider extends ControllerSupport implements OeExamStudentCloudService {
    /**
     *
     */
    private static final long serialVersionUID = 1638873981075522570L;

    @Autowired
    private ExamStudentService examStudentService;

    @Autowired
    private ExamStudentRepo examStudentRepo;

    @Autowired
    private ExamRecordRepo examRecordRepo;

    @Deprecated
    @ApiOperation(value = "同步考生(全部)数据")
    @PostMapping("/sync/all/data")
    public ExamStudentSyncAllDataResp syncExamStudentAllData(@RequestBody ExamStudentSyncAllDataReq req) {
        /*List<ExamStudentInfo> examStudents = ExamStudentBeanConvert.of(req);
        examStudentService.syncExamStudentAllData(examStudents);
        return new ExamStudentSyncAllDataResp();*/
        return null;
    }

    @Deprecated
    @ApiOperation(value = "同步考生(部分)数据")
    @PostMapping("/sync/part/data")
    public ExamStudentSyncPartDataResp syncExamStudentPartData(@RequestBody ExamStudentSyncPartDataReq req) {
        /*List<ExamStudentPartInfo> examStudents = ExamStudentBeanConvert.of(req);
        examStudentService.syncExamStudentPartData(examStudents);
        return new ExamStudentSyncPartDataResp();*/
        return null;
    }

    @Override
    @ApiOperation(value = "根据条件查询考生")
    @PostMapping("/findExamStudentInfoBy")
    public FindExamStudentInfoResp findExamStudentInfoBy(@RequestBody FindExamStudentInfoReq req) {
        FindStudentType findStudentType = req.getFindStudentType();
        if (findStudentType == null) {
            throw new StatusException("ExamStudentCloudServiceProvider-findExamStudentInfoBy-exception", "查询方式不能为空");
        }
        FindExamStudentInfoResp resp = new FindExamStudentInfoResp();

        if (findStudentType == FindStudentType.BY_EXAMID) {
            Long examId = req.getFindStudentBean().getExamId();
            if (examId != null) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByExamId(examId);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        } else if (findStudentType == FindStudentType.BY_EXAMID_AND_IDENTITYNUMBERS) {
            Long examId = req.getFindStudentBean().getExamId();
            List<String> identityNumbes = req.getFindStudentBean().getIdentityNumbers();
            if (examId != null && identityNumbes != null && identityNumbes.size() > 0) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByExamIdAndIdentityNumberIn(examId, identityNumbes);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        } else if (findStudentType == FindStudentType.BY_IDENTITYNUMBERS_AND_ROOTORGID) {
            Long rootOrgId = req.getFindStudentBean().getRootOrgId();
            List<String> identityNumbes = req.getFindStudentBean().getIdentityNumbers();
            if (rootOrgId != null && identityNumbes != null && identityNumbes.size() > 0) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByRootOrgIdAndIdentityNumberIn(rootOrgId, identityNumbes);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        } else if (findStudentType == FindStudentType.BY_EXAMID_AND_STUDENTCODES) {
            Long examId = req.getFindStudentBean().getExamId();
            List<String> studentCodes = req.getFindStudentBean().getStudentCodes();
            if (examId != null && studentCodes != null && studentCodes.size() > 0) {
                List<ExamStudentEntity> examStudents = examStudentRepo.findByExamIdAndStudentCodeIn(examId, studentCodes);
                resp.setExamStudents(buildExamStudentBeans(examStudents));
            }
        }
        return resp;
    }

    @Override
    @ApiOperation(value = "解绑考生学号")
    @PostMapping("/unbindExamStudent")
    public UnbindExamStudentResp unbindExamStudent(UnbindExamStudentReq req) {
        UnbindExamStudentResp resp = new UnbindExamStudentResp();
        if (StringUtils.isNullOrEmpty(req.getIdentityNumber())) {
            resp.setSuccess(false);
            resp.setMsg("身份证号不允许为空");
            return resp;
        }
        if (null == req.getStudentCodeList() || req.getStudentCodeList().isEmpty()) {
            resp.setSuccess(false);
            resp.setMsg("学号不允许为空");
            return resp;
        }
        examStudentRepo.updateExamStudentCode(req.getIdentityNumber(), req.getStudentCodeList());
        resp.setSuccess(true);
        return resp;
    }

    private List<OeExamStudentBean> buildExamStudentBeans(List<ExamStudentEntity> examStudents) {
        List<OeExamStudentBean> examStudentBeans = new ArrayList<OeExamStudentBean>();
        for (ExamStudentEntity examStudent : examStudents) {
            OeExamStudentBean examStudentBean = new OeExamStudentBean();
            examStudentBean.setExamStudentId(examStudent.getExamStudentId());
            examStudentBean.setExamId(examStudent.getExamId());
            examStudentBean.setStudentCode(examStudent.getStudentCode());
            examStudentBean.setStudentName(examStudent.getStudentName());
            examStudentBean.setIdentityNumber(examStudent.getIdentityNumber());
            examStudentBean.setCourseCode(examStudent.getCourseCode());
            examStudentBean.setFinished(examStudent.getFinished());
            examStudentBeans.add(examStudentBean);
        }
        return examStudentBeans;
    }
}