package cn.com.qmth.examcloud.core.oe.admin.api.provider;

import cn.com.qmth.examcloud.core.oe.admin.api.request.UnbindExamStudentReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.UnbindExamStudentResp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamStudentBeanConvert;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamStudentRepo;
import cn.com.qmth.examcloud.global.api.HandleSyncCloudService;
import cn.com.qmth.examcloud.global.api.request.SyncCourseReq;
import cn.com.qmth.examcloud.global.api.request.SyncExamReq;
import cn.com.qmth.examcloud.global.api.request.SyncExamStudentReq;
import cn.com.qmth.examcloud.global.api.request.SyncOrgReq;
import cn.com.qmth.examcloud.global.api.request.SyncSpecialtyReq;
import cn.com.qmth.examcloud.global.api.request.SyncStudentReq;
import cn.com.qmth.examcloud.global.api.request.SyncUserReq;
import cn.com.qmth.examcloud.global.api.response.SyncCourseResp;
import cn.com.qmth.examcloud.global.api.response.SyncExamResp;
import cn.com.qmth.examcloud.global.api.response.SyncExamStudentResp;
import cn.com.qmth.examcloud.global.api.response.SyncOrgResp;
import cn.com.qmth.examcloud.global.api.response.SyncSpecialtyResp;
import cn.com.qmth.examcloud.global.api.response.SyncStudentResp;
import cn.com.qmth.examcloud.global.api.response.SyncUserResp;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * 
 * @author  	chenken
 * @date    	2018年10月29日 下午4:39:24
 * @company 	QMTH
 * @description 数据同步相关接口
 */
@Api(tags = "数据同步相关接口")
@RestController
@RequestMapping("${$rmp.cloud.oe}/exam/dataSync")
public class SyncCloudServiceProvider  extends ControllerSupport implements HandleSyncCloudService {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -4093980356473146740L;

	@Autowired
    private ExamStudentService examStudentService;
	
    @Autowired
    private ExamStudentRepo examStudentRepo;
    
    @Autowired
    private ExamRecordRepo examRecordRepo;

	@ApiOperation(value = "同步考生")
    @PostMapping("/syncExamStudent")
    public SyncExamStudentResp syncExamStudent(@RequestBody SyncExamStudentReq req) {
    	if("deleteByExamId".equals(req.getSyncType())){
    		long examId = req.getExamId();
    		ExamRecordEntity examRecordEntity = new ExamRecordEntity();
    		examRecordEntity.setExamId(examId);
    		long examRecordNum = examRecordRepo.count(Example.of(examRecordEntity));
    		if(examRecordNum > 0){
    			throw new StatusException("syncExamStudent-exception", "该考试已开始,不能删除考生");
    		}
    		examStudentRepo.deleteByExamId(examId);
    	}else if("delete".equals(req.getSyncType())){
    		ExamStudentEntity examStudent = examStudentRepo.findByExamStudentId(req.getId());
    		if(examStudent != null && (examStudent.getFinished() == null || !examStudent.getFinished())){
    			examStudentRepo.delete(examStudent);
    		}
    	}else if("update".equals(req.getSyncType())){
	        ExamStudentInfo examStudent = ExamStudentBeanConvert.of(req);
	        examStudentService.syncExamStudentAllData(Lists.newArrayList(examStudent));
    	}
    	return null;
    }

    @ApiOperation(value = "同步课程")
    @PostMapping("/syncCourse")
    public SyncCourseResp syncCourse(@RequestBody SyncCourseReq req) {
    	Long courseId = req.getId();
    	String courseLevel = req.getLevel();
    	examStudentRepo.updateCourse(courseId, courseLevel);
    	examRecordRepo.updateCourse(courseId, courseLevel);
        return null;
    }

    @ApiOperation(value = "同步学生")
    @PostMapping("/syncStudent")
    public SyncStudentResp syncStudent(@RequestBody SyncStudentReq req) {
    	//考生表和考试记录表
    	long studentId = req.getId();
    	String studentName = req.getName();

//		examStudentRepo.updateStudentNameAndStudentCode(studentId, studentName, studentCode);
//    	examRecordRepo.updateStudentNameAndStudentCode(studentId, studentName, studentCode);
		examStudentRepo.updateStudentName(studentId,studentName);
		examRecordRepo.updateStudentName(studentId, studentName);
		//如果有需要解绑的学号信息，需要对学号进行解绑
		if (null != req.getUnboundStudentCodeList() && !req.getUnboundStudentCodeList().isEmpty()) {
			examStudentRepo.updateExamStudentCode(req.getIdentityNumber(), req.getUnboundStudentCodeList());
		}
    	return null;
    }

    @ApiOperation(value = "同步机构")
    @PostMapping("/syncOrg")
    public SyncOrgResp syncOrg(@RequestBody SyncOrgReq req) {
        return null;
    }
    
    @ApiOperation(value = "同步专业")
    @PostMapping("syncSpecialty")
    public SyncSpecialtyResp syncSpecialty(@RequestBody SyncSpecialtyReq req) {
        return null;
    }

    @ApiOperation(value = "同步考试")
    @PostMapping("syncExam")
    public SyncExamResp syncExam(@RequestBody SyncExamReq req) {
        return null;
    }

	@Override
	public SyncUserResp syncUser(SyncUserReq req) {
		// TODO Auto-generated method stub
		return null;
	}

	
	
}
