/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-29 10:44:05.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service;

import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamOrgSettingsCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;

/**
 * @Description 网考缓存实体转换服务
 * @Author lideyin
 * @Date 2019/8/13 16:00
 * @Version 1.0
 */
public class ExamCacheTransferHelper {

    /**
     * 获取缓存的考试信息
     * @param examId 考试id
     * @param orgId 组织机构id
     * @return ExamBean
     */
    public static ExamBean getCachedExam(Long examId,Long orgId){
        ExamOrgSettingsCacheBean examCacheBean = CacheHelper.getExamOrgSettings(examId,orgId);
        return copyExamBeanFrom(examCacheBean);
    }
    /**
     * 获取缓存的考试信息
     * @param examId 考试id
     * @return ExamBean
     */
    public static ExamBean getCachedExam(Long examId){
        ExamSettingsCacheBean examCacheBean = CacheHelper.getExamSettings(examId);
        return copyExamBeanFrom(examCacheBean);
    }

    /**
     * 获取能在的课程
     * @param courseId 课程id
     * @return CourseBean
     */
    public static CourseBean getCachedCourse(Long courseId){
        CourseCacheBean courseCacheBean = CacheHelper.getCourse(courseId);
        return copyCourseBeanFrom(courseCacheBean);
    }

    private static ExamBean copyExamBeanFrom(ExamOrgSettingsCacheBean examCacheBean) {
        ExamBean resultBean = new ExamBean();
        resultBean.setId(examCacheBean.getId());
        resultBean.setBeginTime(examCacheBean.getBeginTime());
        resultBean.setCode(examCacheBean.getCode());
        resultBean.setDuration(examCacheBean.getDuration());
        resultBean.setEnable(examCacheBean.getEnable());
        resultBean.setEndTime(examCacheBean.getEndTime());
        resultBean.setExamLimit(examCacheBean.getExamLimit());
        resultBean.setExamTimes(examCacheBean.getExamTimes());
        resultBean.setExamType(examCacheBean.getExamType());
        resultBean.setName(examCacheBean.getName());
        resultBean.setRemark(examCacheBean.getRemark());
        resultBean.setRootOrgId(examCacheBean.getRootOrgId());
        return resultBean;
    }

    private static ExamBean copyExamBeanFrom(ExamSettingsCacheBean examCacheBean) {
        ExamBean resultBean = new ExamBean();
        resultBean.setId(examCacheBean.getId());
        resultBean.setBeginTime(examCacheBean.getBeginTime());
        resultBean.setCode(examCacheBean.getCode());
        resultBean.setDuration(examCacheBean.getDuration());
        resultBean.setEnable(examCacheBean.getEnable());
        resultBean.setEndTime(examCacheBean.getEndTime());
        resultBean.setExamLimit(examCacheBean.getExamLimit());
        resultBean.setExamTimes(examCacheBean.getExamTimes());
        resultBean.setExamType(examCacheBean.getExamType());
        resultBean.setName(examCacheBean.getName());
        resultBean.setRemark(examCacheBean.getRemark());
        resultBean.setRootOrgId(examCacheBean.getRootOrgId());
        return resultBean;
    }

    private static CourseBean copyCourseBeanFrom(CourseCacheBean courseCacheBean) {
        CourseBean resultBean = new CourseBean();
        resultBean.setCode(courseCacheBean.getCode());
        resultBean.setEnable(courseCacheBean.getEnable());
        resultBean.setId(courseCacheBean.getId());
        resultBean.setLevel(courseCacheBean.getLevel());
        resultBean.setName(courseCacheBean.getName());
        resultBean.setRootOrgId(courseCacheBean.getRootOrgId());

        return resultBean;
    }
}