/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-15 14:15:19.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service;

import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.*;

import org.springframework.data.domain.Page;

import java.util.List;

/**
 * 考生信息接口
 *
 * @author: fengdesheng
 * @since: 2018/8/15
 */
public interface ExamStudentService {

    /**
     * 同步考生(全部)数据
     *
     * @param examStudents
     * @return
     */
    void syncExamStudentAllData(List<ExamStudentInfo> examStudents);

    /**
     * 同步考生(部分)数据
     *
     * @param examStudents
     * @return
     */
    void syncExamStudentPartData(List<ExamStudentPartInfo> examStudents);

    /**
     * 同步清理考生数据
     *
     * @param examId
     * @return
     */
    void syncRemoveExamStudentByExamId(Long examId);

    /**
     * 查询考生列表(分页)
     *
     * @param query
     * @return
     */
    Page<ExamStudentInfo> getExamStudentListPage(ExamStudentQuery query);
    
    
    public List<ExamStudentInfo> getExamStudentInfoList(ExamStudentQuery query);
    
    /**
     * 查询重考考生列表(分页)
     *
     * @param query
     * @return
     */
    Page<ExamStudentInfo> getReExamineStudentList(ExamStudentQuery query);

    /**
     * 考试概况-按考试完成进度统计
     *
     * @param examId
     * @return
     */
    ExamStudentFinishedStatistic getExamStudentStatisticByFinished(Long examId);

    /**
     * 考试概况-按学习中心统计
     *
     * @param examId
     * @param orgId
     * @return
     */
    List<ExamStudentOrgStatistic> getExamStudentStatisticByOrg(Long examId, Long orgId);

    /**
     * 获取考生信息
     *
     * @param examStudentId
     * @return
     */
    ExamStudentInfo getExamStudentInfo(Long examStudentId);
    /**
     * 查询课程信息
     * @param examId
     * @param orgId
     * @return
     */
	List<Long> findCoursesFromExamStudent(Long examId, Long orgId);

	/**
	 * 设置重考
	 * @param examStudentId
	 */
	public void setReexamine(Long examStudentId);
	/**
	 * 课程进度查询
	 * @param examId
	 * @param courseId
	 */
	public List<CourseProgressInfo> queryCourseProgressInfos(Long examId, Long courseId,String orderColumn);

	/**
	 * @Author lideyin
	 * @Description 获取指定数量的考生数据
	 * @Date 2019/7/18 14:36
	 * @Param [ examId, startId, size]
	 * @return java.util.List<cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo>
	 */
	public List<ExamStudentInfo> getLimitExamStudentList(Long examId,Long startId,Integer size);

}