/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-27 16:27:13.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.bean.examaudit;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.springframework.data.domain.Page;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.core.oe.common.enums.AuditStatus;
import cn.com.qmth.examcloud.core.oe.common.enums.CourseLevel;
import cn.com.qmth.examcloud.core.oe.common.enums.DisciplineType;
import cn.com.qmth.examcloud.core.oe.common.enums.IsSuccess;

/**
 * 类信息转换
 *
 * @author: fengdesheng
 * @since: 2018/8/21
 */
public class ExamAuditEntityConvert {

    public static List<ExamAuditInfo> of(List<HashMap> maps, String examName) {
        if (maps == null || maps.size() == 0) {
            return Lists.newArrayList();
        }
        List<ExamAuditInfo> list = new ArrayList<>();
        for (Map map : maps) {
            try {
                //转换Map数据
                ExamAuditInfo info = new ExamAuditInfo();
                BeanUtils.populate(info, map);
                //转换枚举名称
                info.setCourseLevel(CourseLevel.getCourseLevelTitle(info.getCourseLevel()));
                info.setStatus(AuditStatus.getTitle(info.getStatus()));
                IsSuccess result = IsSuccess.strToEnum(info.getFaceVerifyResult());
                if (result != null) {
                    info.setFaceVerifyResult(result.getDesc());
                }
                info.setExamName(examName);
                
                DisciplineType disciplineType = DisciplineType.tramsformByType(info.getDisciplineType());
                if(disciplineType!=null){
                	info.setDisciplineType(disciplineType.getName());	
                }
                
                list.add(info);
            } catch (Exception e) {
                throw new ExamCloudRuntimeException(e.getMessage(), e.getCause());
            }
        }
        return list;
    }

    public static List<ExamAuditExcel> ofExcel(Page<ExamAuditInfo> page) {
        List<ExamAuditExcel> list = new ArrayList<>();
        if (page == null || page.getContent() == null) {
            return list;
        }
        page.getContent().forEach(e -> {
            ExamAuditExcel excel = new ExamAuditExcel();
            excel.setExamRecordDataId(e.getExamRecordDataId());
            /*DisciplineType disciplineType = DisciplineType.tramsformByType(e.getDisciplineType());
            if(disciplineType!=null){
            	excel.setDisciplineType(disciplineType.getName());
            }*/
            excel.setOrgName(e.getOrgName());
            excel.setDisciplineType(e.getDisciplineType());
            excel.setExamName(e.getExamName());
            excel.setStudentCode(e.getStudentCode());
            excel.setStudentName(e.getStudentName());
            excel.setCourseCode(e.getCourseCode());
            excel.setCourseName(e.getCourseName());
            excel.setCourseLevel(e.getCourseLevel());
            excel.setFaceSuccessCount(e.getFaceSuccessCount());
            excel.setFaceStrangerCount(e.getFaceStrangerCount());
            excel.setFaceTotalCount(e.getFaceTotalCount());
            excel.setFaceSuccessPercent(e.getFaceSuccessPercent());
            list.add(excel);
        });
        return list;
    }

}