/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-24 13:54:59.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.base.utils.DateUtils;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.CourseLevel;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.common.enums.IsSuccess;
import cn.com.qmth.examcloud.core.oe.common.enums.TrueFalse;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordRepo;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.common.service.LocalCacheService;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 类信息转换
 *
 * @author: fengdesheng
 * @since: 2018/8/21
 */
@Component
public class ExamRecordEntityConvert {
	
	@Autowired
	private ExamRecordRepo examRecordRepo;

    @Autowired
    private GainBaseDataService gainBaseDataService;
    public  ExamRecordInfo of(ExamRecordDataEntity recordData,Map<String,Object> cahcheMap) {
        if (recordData == null) {
            return null;
        }
        ExamRecordEntity record = GlobalHelper.getEntity(examRecordRepo,recordData.getExamRecordId(),ExamRecordEntity.class);
        if (record == null) {
            throw new StatusException(Constants.OE_CODE_500, "考试记录基础信息不存在！");
        }
        ExamRecordInfo info = new ExamRecordInfo();
        //封装基础数据
        info.setId(record.getId());
        info.setDataId(recordData.getId());
        info.setExamId(record.getExamId());
        if (record.getExamType() != null) {
            info.setExamType(record.getExamType().name());
        }
        info.setExamStudentId(record.getExamStudentId());
        info.setStudentId(record.getStudentId());
        info.setStudentCode(record.getStudentCode());
        info.setStudentName(record.getStudentName());
        info.setIdentityNumber(record.getIdentityNumber());
        info.setCourseId(record.getCourseId());
        
        long courseId = record.getCourseId();
    	CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);
        
        info.setCourseCode(courseBean.getCode());
        info.setCourseName(courseBean.getName());
        info.setCourseNameAndCode(courseBean.getName() + "(" + courseBean.getCode() + ")");
        info.setCourseLevel(CourseLevel.getCourseLevelTitle(courseBean.getLevel()));
        info.setOrgId(record.getOrgId());
        
        long orgId = record.getOrgId();
        OrgCacheBean orgBean = gainBaseDataService.getOrgBean(orgId);
        info.setOrgName(orgBean.getName());
        info.setRootOrgId(record.getRootOrgId());
        info.setBasePaperId(record.getBasePaperId());
        info.setPaperStructId(record.getPaperStructId());
        info.setInfoCollector(record.getInfoCollector());
        //封装详细数据
        info.setStartTime(recordData.getStartTime());
        info.setEndTime(recordData.getEndTime());
        info.setCleanTime(recordData.getCleanTime());
        info.setIsWarn(recordData.getIsWarn());
        info.setIsAudit(recordData.getIsAudit());
        if (recordData.getStartTime() != null) {
            info.setPaperStartTime(DateUtils.format(recordData.getStartTime()));
        } else {
            info.setPaperStartTime("");
        }
        if (recordData.getEndTime() != null) {
            info.setPaperSubmitTime(DateUtils.format(recordData.getEndTime()));
            info.setIsSubmit("是");
        } else {
            info.setPaperSubmitTime("");
            info.setIsSubmit("否");
        }

        //每次考试持续时间
        if (recordData.getStartTime() != null && recordData.getEndTime() != null) {
            info.setExamTime(recordData.getUsedExamTime() > 0 ? DateUtils.diff(recordData.getUsedExamTime()) : "");
        } else {
            info.setExamTime("");
        }

        //违纪标志
        info.setDisciplineSign(TrueFalse.of(recordData.getIsWarn()));
        info.setIsIllegality(TrueFalse.of(recordData.getIsIllegality()));

        ExamRecordStatus status = recordData.getExamRecordStatus();
        if (status != null) {
            info.setExamRecordStatus(status.name());
        }
        info.setUsedExamTime(recordData.getUsedExamTime());
        info.setExamOrder(recordData.getExamOrder());
        info.setIsReexamine(recordData.getIsReexamine());
        info.setIsContinued(recordData.getIsContinued());
        info.setContinuedCount(recordData.getContinuedCount());
        info.setFaceSuccessCount(recordData.getFaceSuccessCount());
        info.setFaceFailedCount(recordData.getFaceFailedCount());
        info.setFaceStrangerCount(recordData.getFaceStrangerCount());
        info.setFaceTotalCount(recordData.getFaceTotalCount());
        info.setFaceSuccessPercent(recordData.getFaceSuccessPercent());
        info.setBaiduFaceLivenessSuccessPercent(recordData.getBaiduFaceLivenessSuccessPercent());
        IsSuccess result = recordData.getFaceVerifyResult();
        if (result != null) {
            info.setFaceVerifyResult(result.getDesc());
        }
        info.setFaceLandmarkVal(recordData.getFaceLandmarkVal());
        return info;
    }

    public Page<ExamRecordInfo> of(Page<ExamRecordDataEntity> page) {
        Pageable pageable = new PageRequest(page.getNumber(), page.getSize());
        List<ExamRecordDataEntity> entities = page.getContent();
        if (entities == null || entities.size() == 0) {
            return new PageImpl<ExamRecordInfo>(Lists.newArrayList(), pageable, page.getTotalElements());
        }
        //缓存
        Map<String,Object> cahcheMap = new HashMap<String, Object>();
        List<ExamRecordInfo> list = entities.stream().map(entity -> of(entity,cahcheMap)).collect(Collectors.toList());
        return new PageImpl<ExamRecordInfo>(list, pageable, page.getTotalElements());
    }

    
    public List<ExamRecordInfo> of(List<ExamRecordDataEntity> entities) {
    	//缓存
        Map<String,Object> cahcheMap = new HashMap<String, Object>();
        List<ExamRecordInfo> list = entities.stream().map(entity -> of(entity,cahcheMap)).collect(Collectors.toList());
        return list;
    }
}