/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-24 11:22:55.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import cn.com.qmth.examcloud.core.oe.common.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.api.commons.exchange.JsonSerializable;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**
 * 考试记录查询条件类
 *
 * @author: fengdesheng
 * @since: 2018/8/17
 */
@ApiModel
public class ExamRecordQuery implements JsonSerializable {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 4227973258005786636L;
	private Long examRecordDataId;
    @ApiModelProperty("当前页数")
    private Integer pageNo;
    @ApiModelProperty("每页条数")
    private Integer pageSize;
    @ApiModelProperty("学习中心ID")
    private Long orgId;
    @ApiModelProperty("考试ID")
    private Long examId;
    @ApiModelProperty("学号")
    private String studentCode;
    @ApiModelProperty("学生姓名")
    private String studentName;
    @ApiModelProperty("身份证号")
    private String identityNumber;
    @ApiModelProperty("课程ID")
    private Long courseId;
    @ApiModelProperty("课程层次")
    private String courseLevel;
    @ApiModelProperty("考试记录ID")
    private Long examRecordId;
    @ApiModelProperty("审核状态")
    private Boolean isAudit;
    @ApiModelProperty("是否需要审核")
    private Boolean isWarn;
    @ApiModelProperty("是否有陌生人脸")
    private Boolean hasStranger;
    @ApiModelProperty("人脸识别成功率下限")
    private Double faceSuccessPercentLower;
    @ApiModelProperty("人脸识别成功率上限")
    private Double faceSuccessPercentUpper;
    @ApiModelProperty("人脸真实性比率下限")
    private Double livenessSuccessPercentLower;
    @ApiModelProperty("人脸真实性比率上限")
    private Double livenessSuccessPercentUpper;
    @ApiModelProperty("考试记录状态：EXAM_ING、EXAM_END、EXAM_OVERDUE、EXAM_INVALID")
    private List<String> recordStatuses;
    @ApiModelProperty("采集人")
    private String infoCollector;
    @ApiModelProperty("考试开始时间")
    private String startTime;
    @ApiModelProperty("考试结束时间")
    private String endTime;
    
    public ExamRecordQuery addRecordStatus(String recordStatus) {
        if (recordStatuses == null) {
            recordStatuses = new ArrayList<>();
        }
        if (StringUtils.isNotBlank(recordStatus)) {
            recordStatuses.add(recordStatus);
        }
        return this;
    }

    public List<ExamRecordStatus> getRecordStatusList() {
        List<ExamRecordStatus> list = new ArrayList<>();
        if (recordStatuses != null) {
            for (String name : recordStatuses) {
                ExamRecordStatus status = ExamRecordStatus.getByName(name);
                if (status != null) {
                    list.add(status);
                }
            }
        }
        return list;
    }

    public List<String> getRecordStatuses() {
        return recordStatuses;
    }

    public void setRecordStatuses(List<String> recordStatuses) {
        this.recordStatuses = recordStatuses;
    }

    public Integer getPageNo() {
        return pageNo;
    }

    public void setPageNo(Integer pageNo) {
        this.pageNo = pageNo;
    }

    public Integer getPageSize() {
        return pageSize;
    }

    public void setPageSize(Integer pageSize) {
        this.pageSize = pageSize;
    }

    public Long getOrgId() {
        return orgId;
    }

    public void setOrgId(Long orgId) {
        this.orgId = orgId;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public String getStudentCode() {
        return studentCode;
    }

    public void setStudentCode(String studentCode) {
        this.studentCode = studentCode;
    }

    public String getStudentName() {
        return studentName;
    }

    public void setStudentName(String studentName) {
        this.studentName = studentName;
    }

    public String getIdentityNumber() {
        return identityNumber;
    }

    public void setIdentityNumber(String identityNumber) {
        this.identityNumber = identityNumber;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public String getCourseLevel() {
        return courseLevel;
    }

    public void setCourseLevel(String courseLevel) {
        this.courseLevel = courseLevel;
    }

    public Long getExamRecordId() {
        return examRecordId;
    }

    public void setExamRecordId(Long examRecordId) {
        this.examRecordId = examRecordId;
    }

    public Boolean getHasStranger() {
        return hasStranger;
    }

    public void setHasStranger(Boolean hasStranger) {
        this.hasStranger = hasStranger;
    }

    public Double getFaceSuccessPercentLower() {
        return faceSuccessPercentLower;
    }

    public void setFaceSuccessPercentLower(Double faceSuccessPercentLower) {
        this.faceSuccessPercentLower = faceSuccessPercentLower;
    }

    public Double getFaceSuccessPercentUpper() {
        return faceSuccessPercentUpper;
    }

    public void setFaceSuccessPercentUpper(Double faceSuccessPercentUpper) {
        this.faceSuccessPercentUpper = faceSuccessPercentUpper;
    }

    public Double getLivenessSuccessPercentLower() {
		return livenessSuccessPercentLower;
	}

	public void setLivenessSuccessPercentLower(Double livenessSuccessPercentLower) {
		this.livenessSuccessPercentLower = livenessSuccessPercentLower;
	}

	public Double getLivenessSuccessPercentUpper() {
		return livenessSuccessPercentUpper;
	}

	public void setLivenessSuccessPercentUpper(Double livenessSuccessPercentUpper) {
		this.livenessSuccessPercentUpper = livenessSuccessPercentUpper;
	}

	public String getInfoCollector() {
        return infoCollector;
    }

    public void setInfoCollector(String infoCollector) {
        this.infoCollector = infoCollector;
    }

    public String getStartTime() {
        return startTime;
    }

    public void setStartTime(String startTime) {
        this.startTime = startTime;
    }

    public String getEndTime() {
        return endTime;
    }

    public void setEndTime(String endTime) {
        this.endTime = endTime;
    }

	public Long getExamRecordDataId() {
		return examRecordDataId;
	}

	public void setExamRecordDataId(Long examRecordDataId) {
		this.examRecordDataId = examRecordDataId;
	}

	public Boolean getIsAudit() {
		return isAudit;
	}

	public void setIsAudit(Boolean isAudit) {
		this.isAudit = isAudit;
	}

	public Boolean getIsWarn() {
		return isWarn;
	}

	public void setIsWarn(Boolean isWarn) {
		this.isWarn = isWarn;
	}

}