/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-31 17:24:19.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import cn.com.qmth.examcloud.core.oe.admin.service.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordRepo;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import org.apache.commons.lang3.StringUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.netflix.discovery.CommonConstants;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamAuditService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamCaptureService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examcapture.ExamCaptureAuditInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examcapture.ExamCaptureInfo;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamAuditEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamCaptureEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.AuditStatus;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamCaptureRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;

/**
 * 考试抓拍检测相关接口
 *
 * @author: fengdesheng
 * @since: 2018/8/29
 */
@Service
public class ExamCaptureServiceImpl implements ExamCaptureService {
    @Autowired
    private ExamCaptureRepo examCaptureRepo;
    @Autowired
    private ExamAuditService examAuditService;
    @Autowired
    private ExamRecordService examRecordService;
    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;
    @Autowired
    private GainBaseDataService gainBaseDataService;
    @Autowired
    private ExamRecordRepo examRecordRepo;

    @Override
    public ExamCaptureAuditInfo getExamCaptureAuditDetail(Long examRecordDataId) {
        if (examRecordDataId == null) {
            return null;
        }
        ExamRecordDataEntity recordData = GlobalHelper.getEntity(examRecordDataRepo,examRecordDataId,ExamRecordDataEntity.class);
        if (recordData == null) {
            return null;
        }
        ExamRecordEntity examRecord =GlobalHelper.getEntity(examRecordRepo,recordData.getExamRecordId(),ExamRecordEntity.class);

        ExamAuditEntity audit = examAuditService.getExamAuditByExamRecordDataId(examRecordDataId);
        ExamCaptureAuditInfo detail = new ExamCaptureAuditInfo();
        if (audit != null) {
             if (audit.getStatus() != null) {
                 detail.setStatus(audit.getStatus().getName());
             } else {
                 detail.setStatus(AuditStatus.UN_PASS.getName());
             }
             detail.setDisciplineDetail(audit.getDisciplineDetail());
             if(audit.getDisciplineType() != null){
             	detail.setDisciplineType(audit.getDisciplineType().getName());
             }
        }
        
        detail.setExamRecordDataId(examRecordDataId);
        detail.setExamRecordId(examRecord.getId());
        detail.setStudentId(examRecord.getStudentId());
        detail.setStudentCode(examRecord.getStudentCode());
        detail.setStudentName(examRecord.getStudentName());
        detail.setIdentityNumber(examRecord.getIdentityNumber());
        
        CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(examRecord.getCourseId());

        detail.setCourseCode(courseBean.getCode());
        detail.setCourseName(courseBean.getName());
        
        detail.setIsWarn(recordData.getIsWarn());
        detail.setIsAudit(recordData.getIsAudit());
        detail.setIsIllegality(recordData.getIsIllegality());
        detail.setFaceSuccessCount(recordData.getFaceSuccessCount());
        detail.setFaceFailedCount(recordData.getFaceFailedCount());
        detail.setFaceStrangerCount(recordData.getFaceStrangerCount());
        detail.setFaceTotalCount(recordData.getFaceTotalCount());
        detail.setFaceSuccessPercent(recordData.getFaceSuccessPercent());
        detail.setFaceLandmarkVal(recordData.getFaceLandmarkVal());
        detail.setBaiduFaceLivenessSuccessPercent(recordData.getBaiduFaceLivenessSuccessPercent());
        if(recordData.getFaceVerifyResult()!=null){
        	detail.setFaceVerifyResult(recordData.getFaceVerifyResult().name());
        }
        detail.setVirtualCameraNames(getVirtualCameraNames(examRecordDataId));
        return detail;
    }

    /**
     * 获取虚拟摄像头名称
     * @param examRecordDataId
     * @return
     */
    private String getVirtualCameraNames(Long examRecordDataId){
    	 Set<String> virtualCameraNames = new HashSet<String>();
         List<ExamCaptureEntity> examCaptureList = examCaptureRepo.findByExamRecordDataId(examRecordDataId);
         if(examCaptureList != null && examCaptureList.size() >0){
         	for(ExamCaptureEntity examCapture:examCaptureList){
         		String cameraInfos = examCapture.getCameraInfos();
         		if(StringUtils.isNotBlank(cameraInfos)){
         			JSONArray jsonArray;
					try {
						jsonArray = new JSONArray(cameraInfos);						
						for(int i = 0;i<jsonArray.length();i++){
	         				JSONObject jsonObject = (JSONObject) jsonArray.get(i);
	         				if(StringUtils.isBlank(jsonObject.getString("pid"))){
	         					virtualCameraNames.add(jsonObject.getString("name"));
	         				}
	         			}
					} catch (JSONException e) {
						e.printStackTrace();
						throw new StatusException("ExamCaptureService-001", "获取虚拟摄像头名称失败");
					}
         			
         		}
         	}
         }
         StringBuffer sb = new StringBuffer();
         for(String cameraName:virtualCameraNames){
        	 sb.append(cameraName).append(";");
         }
         return sb.toString();
    }

    @Override
    public List<ExamCaptureInfo> getExamCaptureList(Long examRecordDataId) {
        if (examRecordDataId == null) {
            return Lists.newArrayList();
        }
        List<ExamCaptureEntity> examCaptureEntityList = examCaptureRepo.findByExamRecordDataId(examRecordDataId);
        List<ExamCaptureInfo> examCaptureInfoList = new ArrayList<ExamCaptureInfo>();
        for(ExamCaptureEntity examCapture:examCaptureEntityList){
        	examCaptureInfoList.add(buildExamCaptureInfo(examCapture));
        }
        return examCaptureInfoList;
    }

    
    public ExamCaptureInfo buildExamCaptureInfo(ExamCaptureEntity entity) {
        ExamCaptureInfo info = new ExamCaptureInfo();
        info.setId(entity.getId());
        info.setExamRecordDataId(entity.getExamRecordDataId());
        info.setFileUrl(entity.getFileUrl());
        info.setReturnMessage(entity.getFaceCompareResult());
        info.setPass(entity.getIsPass());
        info.setStranger(entity.getIsStranger());
        
        String faceLivenessResult = entity.getFacelivenessResult();
        info.setIsFacelivenessPass(isFaceLivenessPass(faceLivenessResult));
        
        String cameraInfos = entity.getCameraInfos();
        info.setVirtualCameraNames(getVirtualCameraNames(cameraInfos));
        return info;
    }
    
    private String getVirtualCameraNames(String cameraInfos){
    	StringBuffer sb = new StringBuffer();
    	if(StringUtils.isNotBlank(cameraInfos)){
 			JSONArray jsonArray;
			try {
				jsonArray = new JSONArray(cameraInfos);
				for(int i = 0;i<jsonArray.length();i++){
	 				JSONObject jsonObject = (JSONObject) jsonArray.get(i);
	 				if(StringUtils.isBlank(jsonObject.getString("pid"))){
	 					sb.append(jsonObject.getString("name")+",");
	 				}
	 			}
			} catch (JSONException e) {
				e.printStackTrace();
				throw new StatusException("ExamCaptureService-002", "获取虚拟摄像头名称失败");
			}
 			
 			if(sb.lastIndexOf(",")>-1){
 				return sb.substring(0, sb.lastIndexOf(","));
 			}
 		}
    	return sb.toString();
    }
    
    private boolean isFaceLivenessPass(String faceLivenessResult){
    	if(StringUtils.isNotBlank(faceLivenessResult)){
			JSONObject jsonObject;
			try {
				jsonObject = new JSONObject(faceLivenessResult);
				if(jsonObject.has("error_code")&&jsonObject.getInt("error_code") == 0&&jsonObject.has("result")){
					JSONObject resultJson = jsonObject.getJSONObject("result");
					if(resultJson.has("face_liveness")){
						double faceLivenessVal = resultJson.getDouble("face_liveness");
						double baiduFacelivenessThreshold = Double.parseDouble(PropertyHolder.getString("$baidu.faceliveness.threshold"));
						if(faceLivenessVal > baiduFacelivenessThreshold){
							return true;
						}
					}
				}
			} catch (JSONException e) {
				e.printStackTrace();
				return false;
			}
			
    	}
		return false;
	}
    
}