/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-31 09:33:22.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.admin.service.*;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreEntityConvert;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examscore.ExamScoreQuery;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentInfo;
import cn.com.qmth.examcloud.core.oe.common.base.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.base.utils.CommonUtil;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.CourseLevel;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamProperties;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.core.oe.common.enums.MarkingType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.common.service.LocalCacheService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 考试分数相关接口
 *
 * @author: fengdesheng
 * @since: 2018/8/29
 */
@Service
public class ExamScoreServiceImpl implements ExamScoreService {
    @Autowired
    private ExamScoreRepo examScoreRepo;
    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;
    @Autowired
    private ExamStudentService examStudentService;
    @Autowired
    private ExamRecordService examRecordService;
    @Autowired
    private ExamService examService;
    @Autowired
    private GainBaseDataService gainBaseDataService;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private LocalCacheService localCacheService;

    @Override
    public Page<ExamScoreInfo> getExamScoreList(ExamScoreQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试批次！");
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize());

        //获取考试信息
        ExamBean examBean = ExamCacheTransferHelper.getCachedExam(query.getExamId());

        //阅卷方式
        String markingType = gainBaseDataService.getExamProperty(examBean.getId(),ExamProperties.MARKING_TYPE.name());
        
        //获取考生列表
        Page<ExamStudentInfo> page = examStudentService.getExamStudentListPage(ExamScoreEntityConvert.of(query));
        List<ExamStudentInfo> examStudentList = page.getContent();
        if (examStudentList == null || examStudentList.size() == 0) {
            return new PageImpl<ExamScoreInfo>(Lists.newArrayList(), pageable, page.getTotalElements());
        }

        //封装成绩统计结果
        List<ExamScoreInfo> examScoreList = new ArrayList<>();
        examStudentList.forEach(examStudent -> {
            examScoreList.add(of(examStudent,examBean,markingType));
        });
        return new PageImpl<ExamScoreInfo>(examScoreList, pageable, page.getTotalElements());
    }

    /**
     * 封装成绩统计结果
     */
    private ExamScoreInfo of(ExamStudentInfo examStudent,ExamBean examBean,String markingType) {
        ExamScoreInfo examScore = new ExamScoreInfo();
        examScore.setExamId(examStudent.getExamId());
        examScore.setExamName(examBean.getName());
        examScore.setStudentId(examStudent.getStudentId());
        examScore.setStudentCode(examStudent.getStudentCode());
        examScore.setStudentName(examStudent.getStudentName());
        examScore.setIdentityNumber(examStudent.getIdentityNumber());
        
        examScore.setCourseId(examStudent.getCourseId());
        examScore.setCourseCode(examStudent.getCourseCode());
        examScore.setCourseName(examStudent.getCourseName());
        examScore.setCourseNameAndCode(examStudent.getCourseName() + "(" + examStudent.getCourseCode() + ")");
        examScore.setCourseLevel(CourseLevel.getCourseLevelTitle(examStudent.getCourseLevel()));
        
        examScore.setOrgId(examStudent.getOrgId());
        examScore.setOrgCode(examStudent.getOrgCode());
        examScore.setOrgName(examStudent.getOrgName());
        examScore.setInfoCollector(examStudent.getInfoCollector());
        examScore.setSpecialtyName(examStudent.getSpecialtyName());
        examScore.setGrade(examStudent.getGrade());
        Long examTimes = examBean.getExamTimes();
        if(examTimes != null){
        	examScore.setLeftExamTimes(examTimes - examStudent.getNormalExamTimes());
        }
        return setCommomScoreInfo(examScore,examStudent.getFinished(),examStudent.getExamStudentId(),examBean,markingType);
    }

	@Override
	public List<ExamScoreInfo> exportExamScoreList(ExamScoreQuery query) {
		Check.isNull(query, "查询参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试批次！");
        //获取考试信息
        ExamBean examBean = ExamCacheTransferHelper.getCachedExam(query.getExamId());

        //阅卷方式
        String markingType = gainBaseDataService.getExamProperty(examBean.getId(),ExamProperties.MARKING_TYPE.name());
        return getExamStudentInfoListOfScoreExport(query,examBean,markingType);
	}
	
	private List<ExamScoreInfo> getExamStudentInfoListOfScoreExport(ExamScoreQuery query,ExamBean examBean,String markingType) {
		Check.isNull(query, "查询参数不能为空！");
        //查询条件
        StringBuffer sql = new StringBuffer();
        sql.append("select id,exam_student_id,exam_id,course_id,course_code,course_level,finished,student_id,student_code,student_name,identity_number"
        		+ ",info_collector,root_org_id,org_id,paper_type,normal_exam_times,is_reexamine,reexamine_completed"
        		+ ",specialty_code,specialty_name,grade from ec_oe_exam_student where 1=1 ");
        if (query.getOrgId() != null) {
            sql.append(" and org_id="+query.getOrgId());
        }
        if (query.getExamId() != null) {
            sql.append(" and exam_id="+query.getExamId());
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and student_code LIKE '%"+query.getStudentCode()+"%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and student_name LIKE '%"+query.getStudentName()+"%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and identity_number LIKE '%"+query.getIdentityNumber()+"%'");
        }
        if (query.getCourseId() != null) {
            sql.append(" and course_id="+query.getCourseId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and course_level= '"+query.getCourseLevel()+"'");
        }
        if (query.getFinished() != null) {
        	sql.append(" and finished= "+query.getFinished());
        }
        sql.append(" order by update_time desc");
        
        if(query.getStartLimit()!=null && query.getEndLimit() != null){
        	int offset = query.getEndLimit() - query.getStartLimit() + 1;
        	sql.append(" limit "+(query.getStartLimit()-1)+","+offset);
        }
        List<ExamStudentEntity> examStudentList = jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudentEntity>(){
			@Override
			public ExamStudentEntity mapRow(ResultSet rs, int rowNum)throws SQLException {
				return getExamStudentEntityByResultSet(rs);
			}
        });
        
        List<ExamScoreInfo> examScoreInfoList = new ArrayList<ExamScoreInfo>();
        //缓存
        Map<String,Object> cahcheMap = new HashMap<String, Object>();
        for(ExamStudentEntity examStudent:examStudentList){
        	long courseId = examStudent.getCourseId();
        	CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(courseId);
        	
        	long orgId = examStudent.getOrgId();
            OrgCacheBean orgBean = gainBaseDataService.getOrgBean(orgId);

        	ExamScoreInfo examScoreInfo = convertToExamScoreInfo(examStudent,courseBean,orgBean,examBean,markingType);
        	examScoreInfoList.add(examScoreInfo);
        }
        return examScoreInfoList;
	}
	
    private ExamScoreInfo convertToExamScoreInfo(ExamStudentEntity examStudent,CourseBean courseBean,OrgCacheBean orgBean,ExamBean examBean,String markingType) {
        if (examStudent == null) {
            return null;
        }
        ExamScoreInfo examScore = new ExamScoreInfo();
        examScore.setExamId(examStudent.getExamId());
        examScore.setExamName(examBean.getName());
        examScore.setStudentId(examStudent.getStudentId());
        examScore.setStudentCode(examStudent.getStudentCode());
        examScore.setStudentName(examStudent.getStudentName());
        examScore.setIdentityNumber(examStudent.getIdentityNumber());
        examScore.setInfoCollector(examStudent.getInfoCollector());
        
        examScore.setCourseId(examStudent.getCourseId());
        examScore.setCourseCode(examStudent.getCourseCode());
        examScore.setCourseName(courseBean.getName());
        examScore.setCourseNameAndCode(courseBean.getName() + "(" + courseBean.getCode() + ")");
        examScore.setCourseLevel(CourseLevel.getCourseLevelTitle(examStudent.getCourseLevel()));
        
        examScore.setOrgId(examStudent.getOrgId());
        examScore.setOrgCode(orgBean.getCode());
        examScore.setOrgName(orgBean.getName());
        
        examScore.setSpecialtyName(examStudent.getSpecialtyName());
        examScore.setGrade(examStudent.getGrade());

        Long examTimes = examBean.getExamTimes();
        if(examTimes != null){
        	examScore.setLeftExamTimes(examTimes - examStudent.getNormalExamTimes());
        }
        
        return setCommomScoreInfo(examScore,examStudent.getFinished(),examStudent.getExamStudentId(),examBean,markingType);
    }
    
    private ExamScoreInfo setCommomScoreInfo(ExamScoreInfo examScoreInfo,Boolean finished,long examStudentId,ExamBean examBean,String markingType){
    	//查询考试记录
        if(finished != null && finished){
        	//查询考试记录
        	List<ExamRecordDataEntity> examRecordList = examRecordService.getExamRecordListByExamStudentId(examStudentId);
        	examScoreInfo.setIsFinished("是");//是否完成考试
        	examScoreInfo.setIsAbsent("否");//是否缺考
        	//违纪次数
        	examScoreInfo.setDisciplineCount(disciplineCount(examRecordList));
            //提交次数
        	examScoreInfo.setSubmitCount(submitCount(examRecordList));
            //设置相关成绩（总分，主观分，客观分）
            ExamScoreEntity finalExamScore = getFinalEffectiveExamScore(examRecordList,examBean,markingType);
            setFinalExamScore(examScoreInfo,finalExamScore);

            //考试开始时间
            examScoreInfo.setStartTime(getExamRecordStartTime(finalExamScore,examRecordList));
        }else{
        	examScoreInfo.setIsFinished("否");//是否完成考试
        	examScoreInfo.setIsAbsent("是");//是否缺考
        	examScoreInfo.setDisciplineCount("--");//违纪次数
        	examScoreInfo.setSubmitCount("--"); //提交次数
            examScoreInfo.setObjectiveScore("--");//主观分
            examScoreInfo.setSubjectiveScore("--");//客观分
        	examScoreInfo.setFinalExamScore("--");//最终成绩
        	examScoreInfo.setStartTime("--");//考试开始时间
        }
        return examScoreInfo;
    }

    /**
     * 计算得出最终有效成绩
     */
    private ExamScoreEntity getFinalEffectiveExamScore(List<ExamRecordDataEntity> examRecordAllList,ExamBean examBean,String markingType) {
        /*
         * 第一次过滤考试记录:
         * 状态为EXAM_END或EXAM_OVERDUE
         */
        List<ExamRecordDataEntity> firstFilterExamRecordList = examRecordAllList.stream().filter(examRecord -> {
            return examRecord.getExamRecordStatus() == ExamRecordStatus.EXAM_END ||
                    examRecord.getExamRecordStatus() == ExamRecordStatus.EXAM_OVERDUE;
        }).collect(Collectors.toList());

        if (firstFilterExamRecordList == null || firstFilterExamRecordList.size() == 0) {
            return null;
        }

        /*
         * 第二次过滤考试记录:
         * 没有违纪的&&(没有警告或有警告已审核通过的)
         */
        Stream<ExamRecordDataEntity> secondFilterExamRecordStream = firstFilterExamRecordList.stream().filter(examRecord -> {
            return !examRecord.getIsIllegality() && (!examRecord.getIsWarn() || (examRecord.getIsWarn() && examRecord.getIsAudit()));
        });

        List<ExamRecordDataEntity> secondFilterExamRecords = secondFilterExamRecordStream.collect(Collectors.toList());
        if (secondFilterExamRecords == null || secondFilterExamRecords.size() == 0) {
            return null;
        }

        //取出有效记录的成绩
        List<Long> examRecordDataIds = new ArrayList<>();
        for (int i = 0; i < secondFilterExamRecords.size(); i++) {
            examRecordDataIds.add(secondFilterExamRecords.get(i).getId());
        }
        List<ExamScoreEntity> effectiveExamScoreList = examScoreRepo.findByExamRecordDataIdIn(examRecordDataIds);
        if (effectiveExamScoreList == null || effectiveExamScoreList.size() == 0) {
            return null;
        }
        //离线考试
        if("OFFLINE".equals(examBean.getExamType())){
            return effectiveExamScoreList.get(0);
        }else{
            if (markingType.equals(MarkingType.ALL.name()) || markingType.equals(MarkingType.OBJECT_SCORE_MAX.name())) {
             	//全部评阅规则或客观分最高规则:取总分最高
                List<ExamScoreEntity> examScores = effectiveExamScoreList
                         .stream()
                         .sorted((o1, o2) -> o2.getTotalScore().compareTo(o1.getTotalScore()))
                         .collect(Collectors.toList());
                return examScores.get(0);
            } else if (markingType.equals(MarkingType.LAST_SUBMIT.name())) {
                 //最后一次提交规则:取最后一次的成绩
                List<ExamScoreEntity> examScores = effectiveExamScoreList
                         .stream()
                         .sorted((o1, o2) -> o2.getId().compareTo(o1.getId()))
                         .collect(Collectors.toList());
                return examScores.get(0);
            }
        }
        return null;
    }

    /**
     * 给考生最终成绩赋值
     * @param examScoreInfo 考试模型实体
     * @param examScore 考试数据实体
     */
    private void setFinalExamScore(ExamScoreInfo examScoreInfo, final ExamScoreEntity examScore) {
        if (examScore == null) {
            examScoreInfo.setObjectiveScore("0");
            examScoreInfo.setSubjectiveScore("0");
            examScoreInfo.setFinalExamScore("0");
        } else {
            DecimalFormat decimalFormat = new DecimalFormat("#.##");
            examScoreInfo.setObjectiveScore(decimalFormat.format(examScore.getObjectiveScore()));
            examScoreInfo.setSubjectiveScore(decimalFormat.format(examScore.getSubjectiveScore()));
            examScoreInfo.setFinalExamScore(decimalFormat.format(examScore.getTotalScore()));
        }
    }


    /**
     * 考试开始时间
     */
    private String getExamRecordStartTime(ExamScoreEntity examScore,List<ExamRecordDataEntity> examRecordList) {
        if (examScore == null) {
            return "";
        }
        ExamRecordDataEntity examRecordData = null;
        for(ExamRecordDataEntity examRecordDataEntity:examRecordList){
        	if(examRecordDataEntity.getId().equals(examScore.getExamRecordDataId())){
        		examRecordData = examRecordDataEntity;
        		break;
        	}
        }
        if (examRecordData == null) {
            return "";
        }
        if (examRecordData.getStartTime() == null) {
            return "";
        }
        return CommonUtil.getDateStrWithSecond(examRecordData.getStartTime());
    }

    /**
     * 违纪次数
     */
    private String disciplineCount(List<ExamRecordDataEntity> examRecordList) {
        Integer disciplineCount = 0;
        if (examRecordList == null) {
            return disciplineCount.toString();
        }
        for (ExamRecordDataEntity e : examRecordList) {
            if (e.getIsIllegality() != null && e.getIsIllegality()) {
                disciplineCount++;
            }
        }
        return disciplineCount.toString();
    }

    /**
     * 正常交卷次数:无警告或者有警告,有审核,有正常提交时间
     */
    private String submitCount(List<ExamRecordDataEntity> examRecordList) {
        Integer submitCount = 0;
        for (ExamRecordDataEntity e : examRecordList) {
            if (e.getIsWarn() == null) {
                e.setIsWarn(false);
            }
            if (e.getIsAudit() == null) {
                e.setIsAudit(false);
            }
            if (e.getIsWarn() && !e.getIsAudit()) {
                //ignore
            } else {
                if (e.getEndTime() != null) {
                    submitCount++;
                }
            }
        }
        return submitCount.toString();
    }
    
    private ExamStudentEntity getExamStudentEntityByResultSet(ResultSet rs) throws SQLException{
    	ExamStudentEntity examStudentEntity = new ExamStudentEntity();
		examStudentEntity.setId(rs.getLong("id"));
		examStudentEntity.setExamStudentId(rs.getLong("exam_student_id"));
		examStudentEntity.setExamId(rs.getLong("exam_id"));
		examStudentEntity.setCourseId(rs.getLong("course_id"));
		examStudentEntity.setCourseCode(rs.getString("course_code"));
		examStudentEntity.setCourseLevel(rs.getString("course_level"));
		examStudentEntity.setFinished(rs.getBoolean("finished"));
		examStudentEntity.setStudentId(rs.getLong("student_id"));
		examStudentEntity.setStudentCode(rs.getString("student_code"));
		examStudentEntity.setStudentName(rs.getString("student_name"));
		examStudentEntity.setIdentityNumber(rs.getString("identity_number"));
		examStudentEntity.setInfoCollector(rs.getString("info_collector"));
		examStudentEntity.setRootOrgId(rs.getLong("root_org_id"));
		examStudentEntity.setOrgId(rs.getLong("org_id"));
		examStudentEntity.setPaperType(rs.getString("paper_type"));
		examStudentEntity.setNormalExamTimes(rs.getInt("normal_exam_times"));
		examStudentEntity.setIsReExamine(rs.getBoolean("is_reexamine"));
		examStudentEntity.setReExamineCompleted(rs.getBoolean("reexamine_completed"));
		examStudentEntity.setSpecialtyCode(rs.getString("specialty_code"));
		examStudentEntity.setSpecialtyName(rs.getString("specialty_name"));
		examStudentEntity.setGrade(rs.getString("grade"));
		return examStudentEntity;
    }
    
}