/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-16 17:43:37.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.*;
import cn.com.qmth.examcloud.core.oe.common.base.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.oe.common.base.jpa.SqlWrapper;
import cn.com.qmth.examcloud.core.oe.common.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordForMarkingEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamStudentEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.*;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordForMarkingRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamStudentRepo;
import cn.com.qmth.examcloud.core.oe.common.service.GainBaseDataService;
import cn.com.qmth.examcloud.core.oe.common.service.LocalCacheService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamBean;
import cn.com.qmth.examcloud.support.cache.bean.OrgCacheBean;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.query.NativeQuery;
import org.hibernate.transform.Transformers;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

import static cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentMapper.statisticFinishedColumns;
import static cn.com.qmth.examcloud.core.oe.admin.service.bean.examstudent.ExamStudentMapper.statisticOrgColumns;

/**
 * 考生信息接口
 *
 * @author: fengdesheng
 * @since: 2018/8/16
 */
@Service
public class ExamStudentServiceImpl implements ExamStudentService {
    private static final Logger log = LoggerFactory.getLogger(ExamStudentServiceImpl.class);
    @Autowired
    private ExamStudentRepo examStudentRepo;
    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;
    @Autowired
    private ExamRecordRepo examRecordRepo;
    @Autowired
    private ExamRecordForMarkingRepo examRecordForMarkingRepo;
    @Autowired
    private ExamRecordService examRecordService;
    @Autowired
    private EntityManager entityManager;
    @Autowired
    private ExamService examService;
    @Autowired
    private GainBaseDataService gainBaseDataService;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private LocalCacheService localCacheService;

    @Override
    public void syncExamStudentAllData(List<ExamStudentInfo> examStudents) {
        Check.isEmpty(examStudents, "考生信息不能为空！");
        //封装并校验数据
        for (ExamStudentInfo cur : examStudents) {
            Check.isNull(cur.getRootOrgId(), "顶级机构ID不能为空！");
            Check.isNull(cur.getOrgId(), "学习中心ID不能为空！");
            Check.isEmpty(cur.getExamStudentId(), "考生ID不能为空！");
            Check.isNull(cur.getExamId(), "考试ID不能为空！");
            Check.isNull(cur.getCourseId(), "课程ID不能为空！");
            Check.isNull(cur.getStudentId(), "学生ID不能为空！");
            ExamStudentEntity entity = examStudentRepo.findByExamStudentId(cur.getExamStudentId());

            if (entity == null) {
                entity = new ExamStudentEntity();
                entity.setFinished(false);
                entity.setNormalExamTimes(0);
                entity.setIsReExamine(false);
                entity.setReExamineCompleted(false);
                entity.setCreationTime(new Date());
            }
            if (entity.getIsReExamine() == null) {
                entity.setIsReExamine(false);
            }
            entity.setExamStudentId(cur.getExamStudentId());
            entity.setRootOrgId(cur.getRootOrgId());
            entity.setOrgId(cur.getOrgId());
            entity.setExamId(cur.getExamId());
            entity.setStudentId(cur.getStudentId());
            entity.setStudentName(cur.getStudentName());
            entity.setStudentCode(cur.getStudentCode());
            entity.setCourseId(cur.getCourseId());
            entity.setCourseCode(cur.getCourseCode());
            entity.setCourseLevel(cur.getCourseLevel());
            entity.setIdentityNumber(cur.getIdentityNumber());
            entity.setInfoCollector(cur.getInfoCollector());
            entity.setSpecialtyCode(cur.getSpecialtyCode());
            entity.setSpecialtyName(cur.getSpecialtyName());
            entity.setPaperType(cur.getPaperType());
            entity.setGrade(cur.getGrade());
            entity.setUpdateTime(new Date());
            entity.setEnable(cur.getEnable());

            //保存考生
            ExamStudentEntity examStudentEntity = examStudentRepo.save(entity);
            //已完成 考试，更新考试记录表
            if (examStudentEntity.getFinished() != null && examStudentEntity.getFinished()) {
                long examStudentId = examStudentEntity.getExamStudentId();
                String studentName = examStudentEntity.getStudentName();
                String studentCode = examStudentEntity.getStudentCode();
                String infoCollector = examStudentEntity.getInfoCollector();
                examRecordRepo.syncUpdateExamStudentInfo(examStudentId, studentName, studentCode, infoCollector);
            }
        }
    }

    @Override
    public void syncExamStudentPartData(List<ExamStudentPartInfo> examStudents) {
        Check.isEmpty(examStudents, "考生信息不能为空！");
        for (ExamStudentPartInfo info : examStudents) {
            Check.isNull(info.getStudentId(), "学生ID不能为空！");
            Check.isNull(info.getStudentCode(), "学号不能为空！");
            Check.isNull(info.getStudentName(), "学生姓名不能为空！");
            List<ExamStudentEntity> entities = examStudentRepo.findByStudentId(info.getStudentId());
            if (entities == null || entities.size() == 0) {
                continue;
            }
            for (ExamStudentEntity entity : entities) {
                //更新学号、姓名等信息
                entity.setStudentCode(info.getStudentCode());
                entity.setStudentName(info.getStudentName());
            }
            //批量更新
            examStudentRepo.saveAll(entities);
        }
    }

    @Override
    @Transactional
    public void syncRemoveExamStudentByExamId(Long examId) {
        Check.isNull(examId, "考试ID不能为空！");
        SqlWrapper wrapper = new SqlWrapper()
                .delete().from("ec_oe_exam_student").where().eq("exam_id", examId);
        int result = entityManager.createNativeQuery(wrapper.build()).executeUpdate();
        log.debug(String.format("[syncRemoveExamStudentByExamId] examId = %s size = %s", examId, result));
    }

    @Override
    public Page<ExamStudentInfo> getExamStudentListPage(ExamStudentQuery query) {
        Check.isNull(query, "查询参数不能为空！");

        ExamBean examBean = ExamCacheTransferHelper.getCachedExam(query.getExamId());


        StringBuffer sql = new StringBuffer();
        sql.append("select id,exam_student_id,exam_id,course_id,course_code,course_level,finished,student_id,student_code,student_name,identity_number"
                + ",info_collector,root_org_id,org_id,paper_type,normal_exam_times,is_reexamine,reexamine_completed"
                + ",specialty_code,specialty_name,grade from ec_oe_exam_student t1 where 1=1 ");
        sql.append(selectExamStudentConfitionSql(query, examBean.getExamType()));
        sql.append(" order by id desc");
        int currentNum = (query.getPageNo() - 1) * query.getPageSize();
        sql.append(" limit " + currentNum + "," + query.getPageSize());
        List<ExamStudentEntity> examStudentList = jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudentEntity>() {
            @Override
            public ExamStudentEntity mapRow(ResultSet rs, int rowNum) throws SQLException {
                return getExamStudentEntityByResultSet(rs);
            }
        });

        List<ExamStudentInfo> examStudentInfoList = new ArrayList<ExamStudentInfo>();
        //缓存
        Map<String, Object> cahcheMap = new HashMap<String, Object>();
        for (ExamStudentEntity examStudentEntity : examStudentList) {
            ExamStudentInfo examStudentInfo = buildExamStudentInfo(examStudentEntity, cahcheMap, examBean.getExamType());
            examStudentInfoList.add(examStudentInfo);
        }
        cahcheMap.clear();
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize());
        long totalSize = countExamStudent(query, examBean.getExamType());
        return new PageImpl<>(examStudentInfoList, pageable, totalSize);
    }

    private long countExamStudent(ExamStudentQuery query, String examType) {
        //查询条件
        StringBuffer sql = new StringBuffer();
        sql.append("select count(t1.id) from ec_oe_exam_student t1 where 1=1 ");
        sql.append(selectExamStudentConfitionSql(query, examType));
        return jdbcTemplate.queryForObject(sql.toString(), Long.class);
    }

    @Override
    public List<ExamStudentInfo> getExamStudentInfoList(ExamStudentQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        ExamBean examBean = ExamCacheTransferHelper.getCachedExam(query.getExamId());

        //查询条件
        StringBuffer sql = new StringBuffer();
        sql.append("select id,exam_student_id,exam_id,course_id,course_code,course_level,finished,student_id,student_code,student_name,identity_number"
                + ",info_collector,root_org_id,org_id,paper_type,normal_exam_times,is_reexamine,reexamine_completed"
                + ",specialty_code,specialty_name,grade from ec_oe_exam_student t1 where 1=1 ");
        sql.append(selectExamStudentConfitionSql(query, examBean.getExamType()));
        sql.append(" order by id desc");

        List<ExamStudentEntity> examStudentList = jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudentEntity>() {
            @Override
            public ExamStudentEntity mapRow(ResultSet rs, int rowNum) throws SQLException {
                return getExamStudentEntityByResultSet(rs);
            }
        });
        List<ExamStudentInfo> examStudentInfoList = new ArrayList<ExamStudentInfo>();
        //缓存
        Map<String, Object> cahcheMap = new HashMap<String, Object>();
        for (ExamStudentEntity examStudentEntity : examStudentList) {
            ExamStudentInfo examStudentInfo = buildExamStudentInfo(examStudentEntity, cahcheMap, examBean.getExamType());
            examStudentInfoList.add(examStudentInfo);
        }
        cahcheMap.clear();
        return examStudentInfoList;
    }

    private StringBuffer selectExamStudentConfitionSql(ExamStudentQuery query, String examType) {
        StringBuffer sql = new StringBuffer();
        if (query.getOrgId() != null) {
            sql.append(" and org_id=" + query.getOrgId());
        }
        if (query.getExamId() != null) {
            sql.append(" and exam_id = " + query.getExamId());
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and student_code LIKE '%" + query.getStudentCode() + "%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and student_name LIKE '%" + query.getStudentName() + "%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and identity_number LIKE '%" + query.getIdentityNumber() + "%'");
        }
        if (StringUtils.isNotBlank(query.getInfoCollector())) {
            sql.append(" and info_collector LIKE '%" + query.getInfoCollector() + "%'");
        }
        if (query.getCourseId() != null) {
            sql.append(" and course_id=" + query.getCourseId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and course_level= '" + query.getCourseLevel() + "'");
        }
        if (query.getFinished() != null) {
            if (ExamType.ONLINE.name().equals(examType)) {
                sql.append(" and finished = " + query.getFinished());
            } else if (ExamType.OFFLINE.name().equals(examType)) {
                //如果忽略是否上传答案时，只要是已抽题则认为已参加考试
                if (query.getIgnoreUploadOfflineAnswer()!=null && true==query.getIgnoreUploadOfflineAnswer()){
                    sql.append(" and finished = " + query.getFinished());
                }else {
                    if (query.getFinished() == 0) {                    //未抽题
                        sql.append(" and finished = 0");
                    } else if (query.getFinished() == 1) {                //已抽题未上传
                        sql.append(" and finished = 1  and not exists (select id from ec_oe_exam_record_4_marking t2 where t1.exam_student_id = t2.exam_student_id)");
                    } else if (query.getFinished() == 2) {                //已抽题已上传
                        sql.append(" and finished = 1  and  exists (select id from ec_oe_exam_record_4_marking t2 where t1.exam_student_id = t2.exam_student_id)");
                    }
                }

            }
        }
        return sql;
    }

    private ExamStudentInfo buildExamStudentInfo(ExamStudentEntity examStudentEntity, Map<String, Object> cahcheMap, String examType) {
        ExamStudentInfo examStudentInfo = ExamStudentEntityConvert.of(examStudentEntity);
        examStudentInfo.setExamType(examType);
        CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudentInfo.getCourseId());
        examStudentInfo.setCourseName(courseBean.getName());
        examStudentInfo.setCourseCode(courseBean.getCode());
        examStudentInfo.setCourseLevel(CourseLevel.getCourseLevel(courseBean.getLevel()).getTitle());

        OrgCacheBean orgBean = gainBaseDataService.getOrgBean(examStudentInfo.getOrgId());
        examStudentInfo.setOrgCode(orgBean.getCode());
        examStudentInfo.setOrgName(orgBean.getName());

        String photoNumber = localCacheService.getStudentPhotoNumber(cahcheMap, examStudentInfo.getStudentId());
        examStudentInfo.setPhone(photoNumber);//电话号码

        if (ExamType.ONLINE.name().equals(examType)) {
            //完成状态
            examStudentInfo.setFinishedStatus(examStudentEntity.getFinished() ? FinishStatus.已完成.name() : FinishStatus.未完成.name());
        } else if (ExamType.OFFLINE.name().equals(examType)) {
            //离线考试:当前机构是否允许上传附件
            String canUploadAttachment = gainBaseDataService.getExamOrgProperty(examStudentInfo.getExamId(), examStudentInfo.getOrgId(), ExamProperties.CAN_UPLOAD_ATTACHMENT.name());
            if (StringUtils.isNotBlank(canUploadAttachment)) {
                examStudentInfo.setCanUploadAttachment(Boolean.valueOf(canUploadAttachment));
            } else {
                examStudentInfo.setCanUploadAttachment(false);
            }
            //完成状态
            if (!examStudentEntity.getFinished()) {
                examStudentInfo.setFinishedStatus(FinishStatus.未抽题.name());
            } else {
                ExamRecordForMarkingEntity examRecordForMarking = examRecordForMarkingRepo.findTopByExamStudentId(examStudentEntity.getExamStudentId());
                if (examRecordForMarking != null && StringUtils.isNotBlank(examRecordForMarking.getOfflineFileUrl())) {
                    examStudentInfo.setFinishedStatus(FinishStatus.已上传.name());
                    examStudentInfo.setOfflineFileUrl(examRecordForMarking.getOfflineFileUrl());
                } else {
                    examStudentInfo.setFinishedStatus(FinishStatus.已抽题.name());
                }
            }
        }
        return examStudentInfo;
    }

    private ExamStudentEntity getExamStudentEntityByResultSet(ResultSet rs) throws SQLException {
        ExamStudentEntity examStudentEntity = new ExamStudentEntity();
        examStudentEntity.setId(rs.getLong("id"));
        examStudentEntity.setExamStudentId(rs.getLong("exam_student_id"));
        examStudentEntity.setExamId(rs.getLong("exam_id"));
        examStudentEntity.setCourseId(rs.getLong("course_id"));
        examStudentEntity.setCourseCode(rs.getString("course_code"));
        examStudentEntity.setCourseLevel(rs.getString("course_level"));
        examStudentEntity.setFinished(rs.getBoolean("finished"));
        examStudentEntity.setStudentId(rs.getLong("student_id"));
        examStudentEntity.setStudentCode(rs.getString("student_code"));
        examStudentEntity.setStudentName(rs.getString("student_name"));
        examStudentEntity.setIdentityNumber(rs.getString("identity_number"));
        examStudentEntity.setInfoCollector(rs.getString("info_collector"));
        examStudentEntity.setRootOrgId(rs.getLong("root_org_id"));
        examStudentEntity.setOrgId(rs.getLong("org_id"));
        examStudentEntity.setPaperType(rs.getString("paper_type"));
        examStudentEntity.setNormalExamTimes(rs.getInt("normal_exam_times"));
        examStudentEntity.setIsReExamine(rs.getBoolean("is_reexamine"));
        examStudentEntity.setReExamineCompleted(rs.getBoolean("reexamine_completed"));
        examStudentEntity.setSpecialtyCode(rs.getString("specialty_code"));
        examStudentEntity.setSpecialtyName(rs.getString("specialty_name"));
        examStudentEntity.setGrade(rs.getString("grade"));
        return examStudentEntity;
    }

    @Override
    public Page<ExamStudentInfo> getReExamineStudentList(ExamStudentQuery query) {
        //获取考试的默认次数
        ExamBean examBean = ExamCacheTransferHelper.getCachedExam(query.getExamId());

        //封装查询条件
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize());
        StringBuffer sql = new StringBuffer();

        sql.append("select t.id," +
                "t.exam_student_id," +
                "t.exam_id," +
                "t.student_id," +
                "t.student_code," +
                "t.student_name," +
                "t.identity_number," +
                "t.course_id," +
                "t.course_code," +
                "t.course_level," +
                "t.org_id," +
                "t.root_org_id," +
                "t.specialty_name," +
                "t.finished," +
                "t.normal_exam_times from ( ");
        sql.append("SELECT * " +
                " FROM ec_oe_exam_student student" +
                " WHERE exam_id = " + examBean.getId() +
                " AND normal_exam_times >= " + examBean.getExamTimes().longValue() +
                " AND ((is_reexamine = 0 OR is_reexamine is null) OR ( is_reexamine = 1 AND reexamine_completed = 1))" +
                " AND NOT EXISTS (" +
                "	SELECT * FROM ec_oe_exam_record t1" +
                "	LEFT JOIN ec_oe_exam_record_data t2 ON t1.id = t2.exam_record_id" +
                "	WHERE" +
                "		t1.exam_id = " + examBean.getId() + " and student.exam_student_id = t1.exam_student_id" +
                "	AND t2.exam_record_status = 'EXAM_ING'" +
                ") ");

        sql.append(" order by student.id desc ");
        sql.append(") t where 1=1 ");
        if (query.getOrgId() != null) {
            sql.append(" and t.org_id = " + query.getOrgId());
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and t.student_code LIKE '%" + query.getStudentCode() + "%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and t.student_name LIKE '%" + query.getStudentName() + "%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and t.identity_number LIKE '%" + query.getIdentityNumber() + "%'");
        }
        if (query.getCourseId() != null && StringUtils.isNotBlank(query.getCourseId() + "")) {
            sql.append(" and t.course_id=" + query.getCourseId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and t.course_level= '" + query.getCourseLevel() + "'");
        }
        int currentNum = (query.getPageNo() - 1) * query.getPageSize();
        sql.append(" limit " + currentNum + "," + query.getPageSize());

        log.info("重考列表sql:" + sql.toString());
        List<ExamStudentEntity> examStudentList = jdbcTemplate.query(sql.toString(), new RowMapper<ExamStudentEntity>() {
            @Override
            public ExamStudentEntity mapRow(ResultSet rs, int rowNum) throws SQLException {
                ExamStudentEntity examStudentEntity = new ExamStudentEntity();
                examStudentEntity.setId(rs.getLong("id"));
                examStudentEntity.setExamStudentId(rs.getLong("exam_student_id"));
                examStudentEntity.setExamId(rs.getLong("exam_id"));
                examStudentEntity.setStudentId(rs.getLong("student_id"));
                examStudentEntity.setStudentCode(rs.getString("student_code"));
                examStudentEntity.setStudentName(rs.getString("student_name"));
                examStudentEntity.setIdentityNumber(rs.getString("identity_number"));
                examStudentEntity.setCourseId(rs.getLong("course_id"));
                examStudentEntity.setCourseCode(rs.getString("course_code"));
                examStudentEntity.setCourseLevel(rs.getString("course_level"));
                examStudentEntity.setOrgId(rs.getLong("org_id"));
                examStudentEntity.setRootOrgId(rs.getLong("root_org_id"));
                examStudentEntity.setSpecialtyName(rs.getString("specialty_name"));
                examStudentEntity.setFinished(rs.getBoolean("finished"));
                examStudentEntity.setNormalExamTimes(rs.getInt("normal_exam_times"));
                return examStudentEntity;
            }
        });
        long totalSize = countReExamine(query, examBean);
        List<ExamStudentInfo> list = new ArrayList<ExamStudentInfo>();
        Map<String, Object> cahcheMap = new HashMap<String, Object>();
        for (ExamStudentEntity examStudentEntity : examStudentList) {
            ExamStudentInfo examStudentInfo = ExamStudentEntityConvert.of(examStudentEntity);
            OrgCacheBean orgBean = gainBaseDataService.getOrgBean(examStudentInfo.getOrgId());
            examStudentInfo.setOrgName(orgBean.getName());
            examStudentInfo.setOrgCode(orgBean.getCode());
            CourseBean courseBean = ExamCacheTransferHelper.getCachedCourse(examStudentInfo.getCourseId());
            examStudentInfo.setCourseName(courseBean.getName());
            list.add(examStudentInfo);
        }
        return new PageImpl<>(list, pageable, totalSize);
    }

    private Long countReExamine(ExamStudentQuery query, ExamBean examBean) {
        StringBuffer sql = new StringBuffer();
        sql.append("SELECT count(*)" +
                " FROM " +
                "	ec_oe_exam_student student" +
                " WHERE" +
                "	exam_id = " + examBean.getId() +
                " AND normal_exam_times >= " + examBean.getExamTimes().longValue() +
                " AND ((is_reexamine = 0 OR is_reexamine is null) OR ( is_reexamine = 1 AND reexamine_completed = 1))" +
                " AND NOT EXISTS (" +
                "	SELECT * FROM ec_oe_exam_record t1" +
                "	LEFT JOIN ec_oe_exam_record_data t2 ON t1.id = t2.exam_record_id" +
                "	WHERE t1.exam_id = " + examBean.getId() +
                "   AND student.exam_student_id = t1.exam_student_id" +
                "	AND t2.exam_record_status = 'EXAM_ING'" +
                ")");
        if (query.getOrgId() != null) {
            sql.append(" and student.org_id = " + query.getOrgId());
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and student.student_code LIKE '%" + query.getStudentCode() + "%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and student.student_name LIKE '%" + query.getStudentName() + "%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and student.identity_number LIKE '%" + query.getIdentityNumber() + "%'");
        }
        if (query.getCourseId() != null && StringUtils.isNotBlank(query.getCourseId() + "")) {
            sql.append(" and student.course_id=" + query.getCourseId());
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and student.course_level= '" + query.getCourseLevel() + "'");
        }
        return jdbcTemplate.queryForObject(sql.toString(), Long.class);
    }

    @Override
    public ExamStudentFinishedStatistic getExamStudentStatisticByFinished(Long examId) {
        SqlWrapper wrapper = new SqlWrapper()
                .select(statisticFinishedColumns())
                .from("ec_oe_exam_student").as("student")
                .where().eq("student.exam_id", examId);
        Query dataQuery = entityManager.createNativeQuery(wrapper.build());
//        dataQuery.unwrap(SQLQuery.class).setResultTransformer(Transformers.aliasToBean(HashMap.class));
        dataQuery.unwrap(NativeQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        Map<String, BigDecimal> map = (HashMap) dataQuery.getSingleResult();
        ExamStudentFinishedStatistic statistic = new ExamStudentFinishedStatistic();
        if (map != null) {
            if (map.get("finished") == null) {
                statistic.setFinished(0);
            } else {
                statistic.setFinished(map.get("finished").intValue());
            }
            if (map.get("unFinished") == null) {
                statistic.setUnFinished(0);
            } else {
                statistic.setUnFinished(map.get("unFinished").intValue());
            }
        }
        return statistic;
    }

    @Override
    public List<ExamStudentOrgStatistic> getExamStudentStatisticByOrg(Long examId, Long orgId) {
        SqlWrapper wrapper = new SqlWrapper()
                .select(statisticOrgColumns())
                .from("ec_oe_exam_student").as("student")
                .where().eq("student.exam_id", examId);
        if (orgId != null) {
            wrapper.and().eq("student.org_id", orgId);
        }
        wrapper.groupBy("student.org_id").orderBy("student.org_id", false);
        Query dataQuery = entityManager.createNativeQuery(wrapper.build());
//        dataQuery.unwrap(SQLQuery.class).setResultTransformer(Transformers.aliasToBean(HashMap.class));
        dataQuery.unwrap(NativeQuery.class).setResultTransformer(Transformers.ALIAS_TO_ENTITY_MAP);
        List<ExamStudentOrgStatistic> examStudentOrgStatisticList = ExamStudentEntityConvert.ofList(dataQuery.getResultList());
        Map<String, Object> cahcheMap = new HashMap<String, Object>();
        for (ExamStudentOrgStatistic statistic : examStudentOrgStatisticList) {
            OrgCacheBean orgBean = gainBaseDataService.getOrgBean(statistic.getOrgId());
            statistic.setOrgCode(orgBean.getCode());
            statistic.setOrgName(orgBean.getName());
        }
        return examStudentOrgStatisticList;
    }

    @Override
    public ExamStudentInfo getExamStudentInfo(Long examStudentId) {
        ExamStudentEntity entity = examStudentRepo.findByExamStudentId(examStudentId);
        return ExamStudentEntityConvert.of(entity);
    }

    @Override
    public List<Long> findCoursesFromExamStudent(Long examId, Long orgId) {
        String sql = "select course_id from ec_oe_exam_student where exam_id = " + examId;
        if (orgId != null) {
            sql += " and org_id = " + orgId;
        }
        sql += "  group by course_id";
        return jdbcTemplate.queryForList(sql, Long.class);
    }

    @Override
    @Transactional
    public void setReexamine(Long examStudentId) {
        List<ExamRecordDataEntity> examRecordDataList = examRecordService.getExamRecordListByExamStudentId(examStudentId);
        //查询出上一次重考的记录
        Optional<ExamRecordDataEntity> examRecordOptional = examRecordDataList.stream().filter(examRecordData ->
                examRecordData.getExamRecordStatus() != ExamRecordStatus.EXAM_INVALID
                        && examRecordData.getIsReexamine() != null
                        && examRecordData.getIsReexamine()
        ).findFirst();
        ExamRecordDataEntity examRecordData = null;
        if (examRecordOptional.isPresent()) {
            examRecordData = examRecordOptional.get();
        }
        if (examRecordData != null) {
            //将上一次重考的记录设置成无效
            examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_INVALID);
            examRecordDataRepo.save(examRecordData);
            //删除生成的阅卷数据，避免传到阅卷
            ExamRecordForMarkingEntity examRecordForMarking = examRecordForMarkingRepo.findByExamRecordDataId(examRecordData.getId());
            if (examRecordForMarking != null) {
                examRecordForMarkingRepo.delete(examRecordForMarking);
            }
        }
        //在考生信息中设置重考为true
        ExamStudentEntity examStudent = examStudentRepo.findByExamStudentId(examStudentId);
        examStudent.setIsReExamine(true);
        //设置了重考，但考生重考未完成,考生开始重考时，该字段改为true
        examStudent.setReExamineCompleted(false);
        examStudentRepo.save(examStudent);
    }

    @Override
    public List<CourseProgressInfo> queryCourseProgressInfos(Long examId, Long courseId, String orderColumn) {
        if (examId == null) {
            return null;
        }
        if (StringUtils.isBlank(orderColumn)) {
            orderColumn = "all_num";
        }
        String sql = "select *,ROUND(tb.completed_num/tb.all_num,2)*100 completed_proportion from ( " +
                " select " +
                " course_id, " +
                " sum(case when finished = 1 then 1 else 0 end) completed_num, " +
                " sum(case when finished = 0 then 1 else 0 end) no_completed_num, " +
                " count(course_id) all_num" +
                " from ec_oe_exam_student " +
                " where exam_id = " + examId;
        if (courseId != null) {
            sql += " and course_id = " + courseId;
        }
        sql += " group by course_id ) tb ORDER BY " + orderColumn + " desc";

        return jdbcTemplate.query(sql, new RowMapper<CourseProgressInfo>() {
            @Override
            public CourseProgressInfo mapRow(ResultSet rs, int rowNum) throws SQLException {
                CourseProgressInfo courseProgressInfo = new CourseProgressInfo();
                courseProgressInfo.setCourseId(rs.getLong("course_id"));
                courseProgressInfo.setCompletedNum(rs.getInt("completed_num"));
                courseProgressInfo.setNoCompletedNum(rs.getInt("no_completed_num"));
                courseProgressInfo.setAllNum(rs.getInt("all_num"));
                courseProgressInfo.setCompletedProportion(rs.getDouble("completed_proportion"));
                return courseProgressInfo;
            }
        });
    }

    @Override
    public List<ExamStudentInfo> getLimitExamStudentList(Long examId, Long startId, Integer size) {
        List<ExamStudentInfo> resultList = new ArrayList<>();
        List<ExamStudentEntity> studentEntityList = examStudentRepo.getLimitExamStudentList(examId, startId, size);
        for (ExamStudentEntity se : studentEntityList) {
            ExamStudentInfo info = new ExamStudentInfo();
            info.setId(se.getId());
            info.setExamStudentId(se.getExamStudentId());
            info.setExamId(se.getExamId());
            info.setCourseId(se.getCourseId());
            info.setCourseCode(se.getCourseCode());
            info.setCourseLevel(se.getCourseLevel());
            info.setFinished(se.getFinished());
            info.setStudentId(se.getStudentId());
            info.setStudentCode(se.getStudentCode());
            info.setStudentName(se.getStudentName());
            info.setIdentityNumber(se.getIdentityNumber());
            info.setInfoCollector(se.getInfoCollector());
            info.setRootOrgId(se.getRootOrgId());
            info.setOrgId(se.getOrgId());
            info.setPaperType(se.getPaperType());
            info.setNormalExamTimes(se.getNormalExamTimes());
            info.setIsReExamine(se.getIsReExamine());
            info.setReExamineCompleted(se.getReExamineCompleted());
            info.setSpecialtyCode(se.getSpecialtyCode());
            info.setSpecialtyName(se.getSpecialtyName());
            info.setGrade(se.getGrade());
            resultList.add(info);
        }
        return resultList;
    }

}